#
#    Copyright (c) 2013 Mirantis, Inc.
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import collections
import filecmp
import io
import json
import logging
import os
import re
import shutil
import sys
import tempfile
from unittest import mock

import fixtures
from keystoneclient import fixture
from keystoneclient.fixture import v2 as ks_v2_fixture
from keystoneclient.fixture import v3 as ks_v3_fixture
from oslo_log import handlers
from oslo_log import log
import requests_mock
from testtools import matchers

from muranoclient.apiclient import exceptions
from muranoclient.common import exceptions as common_exceptions
from muranoclient.common import utils
import muranoclient.shell
from muranoclient.tests.unit import base
from muranoclient.tests.unit import test_utils
from muranoclient.v1 import shell as v1_shell

make_pkg = test_utils.make_pkg

FIXTURE_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                           'fixture_data'))
# RESULT_PACKAGE = os.path.join(FIXTURE_DIR, 'test-app.zip')

FAKE_ENV = {'OS_USERNAME': 'username',
            'OS_PASSWORD': 'password',
            'OS_TENANT_NAME': 'tenant_name',
            'OS_AUTH_URL': 'http://no.where/v2.0'}

FAKE_ENV2 = {'OS_USERNAME': 'username',
             'OS_PASSWORD': 'password',
             'OS_TENANT_ID': 'tenant_id',
             'OS_AUTH_URL': 'http://no.where/v2.0'}

FAKE_ENV_v3 = {'OS_USERNAME': 'username',
               'OS_PASSWORD': 'password',
               'OS_TENANT_ID': 'tenant_id',
               'OS_USER_DOMAIN_NAME': 'domain_name',
               'OS_AUTH_URL': 'http://no.where/v3'}


def _create_ver_list(versions):
    return {'versions': {'values': versions}}


class TestArgs(object):
    package_version = ''
    murano_repo_url = 'http://127.0.0.1'
    exists_action = ''
    dep_exists_action = ''
    is_public = False
    categories = []


class ShellTest(base.TestCaseShell):

    def make_env(self, exclude=None, fake_env=FAKE_ENV):
        env = dict((k, v) for k, v in fake_env.items() if k != exclude)
        self.useFixture(fixtures.MonkeyPatch('os.environ', env))


class ShellCommandTest(ShellTest):

    _msg_no_tenant_project = ('You must provide a project name or project'
                              ' id via --os-project-name, --os-project-id,'
                              ' env[OS_PROJECT_ID] or env[OS_PROJECT_NAME].'
                              ' You may use os-project and os-tenant'
                              ' interchangeably.',)

    def setUp(self):
        super(ShellCommandTest, self).setUp()

        def get_auth_endpoint(bound_self, args):
            return ('test', {})
        self.useFixture(fixtures.MonkeyPatch(
            'muranoclient.shell.MuranoShell._get_endpoint_and_kwargs',
            get_auth_endpoint))
        self.client = mock.MagicMock()

        # To prevent log descriptors from being closed during
        # shell tests set a custom StreamHandler
        self.logger = log.getLogger(None).logger
        self.logger.level = logging.DEBUG
        self.color_handler = handlers.ColorHandler(sys.stdout)
        self.logger.addHandler(self.color_handler)

    def tearDown(self):
        super(ShellTest, self).tearDown()
        self.logger.removeHandler(self.color_handler)

    def shell(self, argstr, exitcodes=(0,)):
        orig = sys.stdout
        orig_stderr = sys.stderr
        try:
            sys.stdout = io.StringIO()
            sys.stderr = io.StringIO()
            _shell = muranoclient.shell.MuranoShell()
            _shell.main(argstr.split())
        except SystemExit:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            self.assertIn(exc_value.code, exitcodes)
        finally:
            stdout = sys.stdout.getvalue()
            sys.stdout.close()
            sys.stdout = orig
            stderr = sys.stderr.getvalue()
            sys.stderr.close()
            sys.stderr = orig_stderr
        return (stdout, stderr)

    def register_keystone_discovery_fixture(self, mreq):
        v2_url = "http://no.where/v2.0"
        v2_version = fixture.V2Discovery(v2_url)
        mreq.register_uri('GET', v2_url, json=_create_ver_list([v2_version]),
                          status_code=200)

    def register_keystone_token_fixture(self, mreq):
        v2_token = ks_v2_fixture.Token(token_id='token')
        service = v2_token.add_service('application-catalog')
        service.add_endpoint('http://no.where', region='RegionOne')
        mreq.register_uri('POST',
                          'http://no.where/v2.0/tokens',
                          json=v2_token,
                          status_code=200)

    def test_help_unknown_command(self):
        self.assertRaises(exceptions.CommandError, self.shell, 'help foofoo')

    def test_help(self):
        required = [
            r'.*?^usage: murano',
            r'.*?^\s+package-create\s+Create an application package.',
            r'.*?^See "murano help COMMAND" for help on a specific command',
        ]
        stdout, stderr = self.shell('help')
        for r in required:
            self.assertThat((stdout + stderr),
                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_help_on_subcommand(self):
        required = [
            '.*?^usage: murano package-create',
            '.*?^Create an application package.',
        ]
        stdout, stderr = self.shell('help package-create')
        for r in required:
            self.assertThat((stdout + stderr),
                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_help_no_options(self):
        required = [
            r'.*?^usage: murano',
            r'.*?^\s+package-create\s+Create an application package',
            r'.*?^See "murano help COMMAND" for help on a specific command',
        ]
        stdout, stderr = self.shell('')
        for r in required:
            self.assertThat((stdout + stderr),
                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_no_username(self):
        required = ('You must provide a username via either --os-username or '
                    'env[OS_USERNAME] or a token via --os-auth-token or '
                    'env[OS_AUTH_TOKEN]',)
        self.make_env(exclude='OS_USERNAME')
        try:
            self.shell('package-list')
        except exceptions.CommandError as message:
            self.assertEqual(required, message.args)
        else:
            self.fail('CommandError not raised')

    def test_no_tenant_name(self):
        required = self._msg_no_tenant_project
        self.make_env(exclude='OS_TENANT_NAME')
        try:
            self.shell('package-list')
        except exceptions.CommandError as message:
            self.assertEqual(required, message.args)
        else:
            self.fail('CommandError not raised')

    def test_no_tenant_id(self):
        required = self._msg_no_tenant_project
        self.make_env(exclude='OS_TENANT_ID', fake_env=FAKE_ENV2)
        try:
            self.shell('package-list')
        except exceptions.CommandError as message:
            self.assertEqual(required, message.args)
        else:
            self.fail('CommandError not raised')

    def test_no_auth_url(self):
        required = ('You must provide an auth url'
                    ' via either --os-auth-url or via env[OS_AUTH_URL]',)
        self.make_env(exclude='OS_AUTH_URL')
        try:
            self.shell('package-list')
        except exceptions.CommandError as message:
            self.assertEqual(required, message.args)
        else:
            self.fail('CommandError not raised')

    @mock.patch('muranoclient.v1.packages.PackageManager')
    @requests_mock.mock()
    def test_package_list(self, mock_package_manager, m_requests):
        self.client.packages = mock_package_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('package-list')
        self.client.packages.filter.assert_called_once_with(
            include_disabled=False,
            owned=False)

    @mock.patch('muranoclient.v1.packages.PackageManager')
    @requests_mock.mock()
    def test_package_list_with_limit(self, mock_package_manager, m_requests):
        self.client.packages = mock_package_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('package-list --limit 10')
        self.client.packages.filter.assert_called_once_with(
            include_disabled=False,
            limit=10,
            owned=False)

    @mock.patch('muranoclient.v1.packages.PackageManager')
    @requests_mock.mock()
    def test_package_list_with_marker(self, mock_package_manager, m_requests):
        self.client.packages = mock_package_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('package-list --marker 12345')
        self.client.packages.filter.assert_called_once_with(
            include_disabled=False,
            marker='12345',
            owned=False)

    @mock.patch('muranoclient.v1.packages.PackageManager')
    @requests_mock.mock()
    def test_package_list_with_name(self, mock_package_manager, m_requests):
        self.client.packages = mock_package_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('package-list --name mysql')
        self.client.packages.filter.assert_called_once_with(
            name='mysql',
            include_disabled=False,
            owned=False)

    @mock.patch('muranoclient.v1.packages.PackageManager')
    @requests_mock.mock()
    def test_package_list_with_fqn(self, mock_package_manager, m_requests):
        self.client.packages = mock_package_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('package-list --fqn mysql')
        self.client.packages.filter.assert_called_once_with(
            fqn='mysql',
            include_disabled=False,
            owned=False)

    @mock.patch('muranoclient.v1.packages.PackageManager')
    @requests_mock.mock()
    def test_package_show(self, mock_package_manager, m_requests):
        self.client.packages = mock_package_manager()
        mock_package = mock.MagicMock()
        mock_package.class_definitions = ''
        mock_package.categories = ''
        mock_package.tags = ''
        mock_package.description = ''
        self.client.packages.get.return_value = mock_package
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('package-show 1234')
        self.client.packages.get.assert_called_with('1234')

    @mock.patch('muranoclient.v1.packages.PackageManager')
    @requests_mock.mock()
    def test_package_update(self, mock_package_manager, m_requests):
        self.client.packages = mock_package_manager()
        mock_package = mock.MagicMock()
        mock_package.class_definitions = ''
        mock_package.categories = ''
        mock_package.tags = ''
        mock_package.description = ''
        self.client.packages.get.return_value = mock_package

        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)

        self.shell('package-update 123 --is-public true')
        self.shell('package-update 123 --is-public false')
        self.shell('package-update 123 --is-public false --enabled t')
        self.shell('package-update 123 --name foo --description bar')
        self.shell('package-update 123 --tags a')
        self.shell('package-update 123 --tags a ' +
                   '--is-public f --enabled f ' +
                   '--name foo ' +
                   '--description bar',)
        self.client.packages.update.assert_has_calls([
            mock.call('123', {'is_public': True}),
            mock.call('123', {'is_public': False}),
            mock.call('123', {'enabled': True, 'is_public': False}),
            mock.call('123', {'name': 'foo', 'description': 'bar'}),
            mock.call('123', {'tags': ['a']}),
            mock.call('123', {
                'tags': ['a'],
                'is_public': False,
                'enabled': False,
                'name': 'foo',
                'description': 'bar',
            }),
        ])

    @mock.patch('muranoclient.v1.packages.PackageManager')
    @requests_mock.mock()
    def test_package_delete(self, mock_package_manager, m_requests):
        self.client.packages = mock_package_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('package-delete 1234 4321')
        self.client.packages.delete.assert_has_calls([
            mock.call('1234'), mock.call('4321')])
        self.assertEqual(2, self.client.packages.delete.call_count)

    @mock.patch('muranoclient.v1.sessions.SessionManager')
    @requests_mock.mock()
    def test_environment_session_create(self, mock_manager, m_requests):
        self.client.sessions = mock_manager()
        self.client.sessions.configure.return_value.id = '123'
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('environment-session-create 1234')
        self.client.sessions.configure.assert_has_calls([
            mock.call('1234')])

    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
    @requests_mock.mock()
    def test_environment_create(self, mock_manager, m_requests):
        self.client.environments = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)

        self.shell('environment-create foo')
        self.client.environments.create.assert_has_calls(
            [mock.call({'name': 'foo', 'region': None})])
        self.client.environments.create.reset_mock()
        self.shell('environment-create --join-net 123 foo --region RegionOne')
        cc = self.client.environments.create
        expected_call = mock.call({
            'defaultNetworks': {
                'environment': {
                    'internalNetworkName': '123',
                    '?': {
                        'type': 'io.murano.resources.ExistingNeutronNetwork',
                        'id': mock.ANY
                    }
                },
                'flat': None
            },
            'name': 'foo',
            'region': 'RegionOne'
        })
        self.assertEqual(expected_call, cc.call_args)

    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
    @requests_mock.mock()
    def test_environment_list(self, mock_manager, m_requests):
        self.client.environments = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)

        self.shell('environment-list')
        self.client.environments.list.assert_called_once_with(False, None)

        self.client.environments.list.reset_mock()
        self.shell('environment-list --all-tenants')
        self.client.environments.list.assert_called_once_with(True, None)

        self.client.environments.list.reset_mock()
        self.shell('environment-list --tenant ABC')
        self.client.environments.list.assert_called_once_with(False, 'ABC')

    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
    @requests_mock.mock()
    def test_environment_delete(self, mock_manager, m_requests):
        self.client.environments = mock_manager()
        self.client.environments.find.return_value.id = '123'
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('environment-delete env1')
        self.client.environments.find.assert_has_calls([
            mock.call(name='env1')
        ])
        self.client.environments.delete.assert_has_calls([
            mock.call('123', False)
        ])

    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
    @requests_mock.mock()
    def test_environment_delete_with_abandon(self, mock_manager, m_requests):
        self.client.environments = mock_manager()
        self.client.environments.find.return_value.id = '123'
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('environment-delete env1 --abandon')
        self.client.environments.find.assert_has_calls([
            mock.call(name='env1')
        ])
        self.client.environments.delete.assert_has_calls([
            mock.call('123', True)
        ])

    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
    @requests_mock.mock()
    def test_environment_rename(self, mock_manager, m_requests):
        self.client.environments = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('environment-rename old-name-or-id new-name')
        self.client.environments.find.assert_called_once_with(
            name='old-name-or-id')
        self.assertEqual(1, self.client.environments.update.call_count)

    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
    @requests_mock.mock()
    def test_environment_show(self, mock_manager, m_requests):
        self.client.environments = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('environment-show env-id-or-name')
        self.client.environments.find.assert_called_once_with(
            name='env-id-or-name')

    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
    @requests_mock.mock()
    def test_environment_model_show_basic(self, mock_manager, m_requests):
        self.client.environments = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('environment-model-show env-id')
        self.client.environments.get_model.assert_called_once_with(
            'env-id', '/', None)

    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
    @requests_mock.mock()
    def test_environment_model_show_full(self, mock_manager, m_requests):
        self.client.environments = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('environment-model-show env-id '
                   '--path /path '
                   '--session-id sess-id')
        self.client.environments.get_model.assert_called_once_with(
            'env-id', '/path', 'sess-id')

    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
    @requests_mock.mock()
    def test_environment_model_show_path_with_q(self, mock_manager,
                                                m_requests):
        self.client.environments = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('environment-model-show env-id '
                   '--path /?/path')
        self.client.environments.get_model.assert_called_once_with(
            'env-id', '/%3F/path', None)

    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
    @requests_mock.mock()
    def test_environment_model_edit(self, mock_manager, m_requests):
        self.client.environments = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.client.environments.get_model.return_value = {'name': "foo"}
        temp_file = tempfile.NamedTemporaryFile(prefix="murano-test", mode='w')
        patch = [{'op': 'replace', 'path': '/name', 'value': 'dummy'}]
        json.dump(patch, temp_file)
        temp_file.file.flush()

        self.shell('environment-model-edit env-id {0} '
                   '--session-id sess-id'.format(temp_file.name))
        self.client.environments.update_model.assert_called_once_with(
            'env-id', patch, 'sess-id')

    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
    @mock.patch('muranoclient.v1.sessions.SessionManager')
    @requests_mock.mock()
    def test_environment_deploy(self, mock_manager, env_manager, m_requests):
        self.client.sessions = mock_manager()
        self.client.environments = env_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('environment-deploy 12345 --session-id 54321')
        self.client.sessions.deploy.assert_called_once_with(
            '12345', '54321')

    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
    @requests_mock.mock()
    def test_environment_show_session(self, mock_manager, m_requests):
        self.client.environments = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('environment-show 12345 --session-id 12345')
        self.client.environments.get.assert_called_once_with(
            12345, session_id='12345')

    @mock.patch('muranoclient.v1.actions.ActionManager')
    @requests_mock.mock()
    def test_environment_action_call(self, mock_manager, m_requests):
        self.client.actions = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('environment-action-call 12345 --action-id 54321')
        self.client.actions.call.assert_called_once_with(
            '12345', '54321', arguments={})

    @mock.patch('muranoclient.v1.actions.ActionManager')
    @requests_mock.mock()
    def test_environment_action_call_args(self, mock_manager, m_requests):
        self.client.actions = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell("""environment-action-call 12345 --action-id 54321
                   --arguments foo=bar
                   dictArg={"key1":"value1","key2":"value2"}
                   listArg=["item1","item2","item3"]
                   nullArg=null
                   stringArg="null"
                   intArg=5
                   compoundArg=["foo",14,{"key1":null,"key2":8}]""")
        self.client.actions.call.assert_called_once_with(
            '12345', '54321', arguments={
                'foo': 'bar',
                'dictArg': {u'key1': u'value1', u'key2': u'value2'},
                'listArg': [u'item1', u'item2', u'item3'],
                'nullArg': None,
                'stringArg': u'null',
                'intArg': 5,
                'compoundArg': [u'foo', 14, {u'key1': None, u'key2': 8}]
            })

    @mock.patch('muranoclient.v1.actions.ActionManager')
    @requests_mock.mock()
    def test_environment_action_get_result(self, mock_manager, m_requests):
        self.client.actions = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('environment-action-get-result 12345 --task-id 54321')
        self.client.actions.get_result.assert_called_once_with(
            '12345', '54321')

    @mock.patch('muranoclient.v1.static_actions.StaticActionManager')
    @requests_mock.mock()
    def test_static_action_call_basic(self, mock_manager, m_requests):
        self.client.static_actions = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('static-action-call class.name method.name')
        self.client.static_actions.call.assert_called_once_with({
            "className": 'class.name',
            "methodName": 'method.name',
            "packageName": None,
            "classVersion": '=0',
            "parameters": {}
        })

    @mock.patch('muranoclient.v1.static_actions.StaticActionManager')
    @requests_mock.mock()
    def test_static_action_call_full(self, mock_manager, m_requests):
        self.client.static_actions = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('static-action-call class.name method.name '
                   '--package-name package.name --class-version ">1"')
        self.client.static_actions.call.assert_called_once_with({
            "className": 'class.name',
            "methodName": 'method.name',
            "packageName": 'package.name',
            "classVersion": '">1"',
            "parameters": {}
        })

    @mock.patch('muranoclient.v1.static_actions.StaticActionManager')
    @requests_mock.mock()
    def test_static_action_call_string_args(self, mock_manager, m_requests):
        self.client.static_actions = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('static-action-call class.name method.name '
                   '--arguments food=spam parrot=dead')
        self.client.static_actions.call.assert_called_once_with({
            "className": 'class.name',
            "methodName": 'method.name',
            "packageName": None,
            "classVersion": '=0',
            "parameters": {'food': 'spam', 'parrot': 'dead'}
        })

    @mock.patch('muranoclient.v1.static_actions.StaticActionManager')
    @requests_mock.mock()
    def test_static_action_call_json_args(self, mock_manager, m_requests):
        self.client.static_actions = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell("""static-action-call class.name method.name
                   --arguments
                   dictArg={"key1":"value1","key2":"value2"}
                   listArg=["item1","item2","item3"]
                   nullArg=null
                   stringArg="null"
                   intArg=5
                   compoundArg=["foo",14,{"key1":null,"key2":8}]""")
        self.client.static_actions.call.assert_called_once_with({
            "className": 'class.name',
            "methodName": 'method.name',
            "packageName": None,
            "classVersion": '=0',
            "parameters": {
                'dictArg': {u'key1': u'value1', u'key2': u'value2'},
                'listArg': [u'item1', u'item2', u'item3'],
                'nullArg': None,
                'stringArg': u'null',
                'intArg': 5,
                'compoundArg': [u'foo', 14, {u'key1': None, u'key2': 8}]
            }
        })

    @mock.patch('muranoclient.v1.schemas.SchemaManager')
    @requests_mock.mock()
    def test_class_schema(self, mock_manager, m_requests):
        self.client.schemas = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('class-schema class.name')
        self.client.schemas.get.assert_called_once_with(
            'class.name', [],
            package_name=None,
            class_version='=0'
        )

    @mock.patch('muranoclient.v1.schemas.SchemaManager')
    @requests_mock.mock()
    def test_class_schema_with_methods(self, mock_manager, m_requests):
        self.client.schemas = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('class-schema class.name method1 method2')
        self.client.schemas.get.assert_called_once_with(
            'class.name', ['method1', 'method2'],
            package_name=None,
            class_version='=0'
        )

    @mock.patch('muranoclient.v1.schemas.SchemaManager')
    @requests_mock.mock()
    def test_class_schema_full(self, mock_manager, m_requests):
        self.client.schemas = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('class-schema class.name method1 method2 '
                   '--class-version >1.2.3 --package-name foo.bar')
        self.client.schemas.get.assert_called_once_with(
            'class.name', ['method1', 'method2'],
            package_name='foo.bar',
            class_version='>1.2.3'
        )

    @mock.patch('muranoclient.v1.templates.EnvTemplateManager')
    @requests_mock.mock()
    def test_env_template_delete(self, mock_manager, m_requests):
        self.client.env_templates = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('env-template-delete env1 env2')
        self.client.env_templates.delete.assert_has_calls([
            mock.call('env1'), mock.call('env2')])

    @mock.patch('muranoclient.v1.templates.EnvTemplateManager')
    @requests_mock.mock()
    def test_env_template_create(self, mock_manager, m_requests):
        self.client.env_templates = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('env-template-create env-name')
        self.client.env_templates.create.assert_called_once_with(
            {'name': 'env-name', 'is_public': False})

    @mock.patch('muranoclient.v1.templates.EnvTemplateManager')
    @requests_mock.mock()
    def test_env_template_create_public(self, mock_manager, m_requests):
        self.client.env_templates = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('env-template-create --is-public env-name')
        self.client.env_templates.create.assert_called_once_with(
            {'name': 'env-name', 'is_public': True})

    @mock.patch('muranoclient.v1.templates.EnvTemplateManager')
    @requests_mock.mock()
    def test_env_template_show(self, mock_manager, m_requests):
        self.client.env_templates = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('env-template-show env-id')
        self.client.env_templates.get.assert_called_once_with('env-id')

    @mock.patch('muranoclient.v1.templates.EnvTemplateManager')
    @requests_mock.mock()
    def test_env_template_create_env(self, mock_manager, m_requests):
        self.client.env_templates = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('env-template-create-env env-id env-name')
        self.client.env_templates.create_env.\
            assert_called_once_with('env-id', {"name": 'env-name'})

    @mock.patch('muranoclient.v1.templates.EnvTemplateManager')
    @requests_mock.mock()
    def test_env_template_create_env_with_region(self, mock_manager,
                                                 m_requests):
        self.client.env_templates = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('env-template-create-env env-id env-name --region Region')
        self.client.env_templates.create_env.\
            assert_called_once_with('env-id', {"name": 'env-name',
                                               "region": 'Region'})

    @mock.patch('muranoclient.v1.templates.EnvTemplateManager')
    @requests_mock.mock()
    def test_env_template_clone(self, mock_manager, m_requests):
        self.client.env_templates = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('env-template-clone env-id env-name')
        self.client.env_templates.clone.assert_called_once_with(
            'env-id', 'env-name')

    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
    @mock.patch('muranoclient.v1.deployments.DeploymentManager')
    @requests_mock.mock()
    def test_deployments_show(self, mock_deployment_manager, mock_env_manager,
                              m_requests):
        self.client.deployments = mock_deployment_manager()
        self.client.environments = mock_env_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('deployment-list env-id-or-name')
        self.client.environments.find.assert_called_once_with(
            name='env-id-or-name')
        self.assertEqual(1, self.client.deployments.list.call_count)

    @mock.patch('muranoclient.v1.deployments.DeploymentManager')
    @requests_mock.mock()
    def test_deployments_list_all_environments(self, mock_deployment_manager,
                                               m_requests):
        self.client.deployments = mock_deployment_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        self.shell('deployment-list --all-environments')
        self.client.deployments.list.assert_called_once_with(
            None, True)

    @mock.patch('muranoclient.v1.deployments.DeploymentManager')
    @requests_mock.mock()
    def test_deployments_list_negative(self, mock_deployment_manager,
                                       m_requests):
        self.client.deployments = mock_deployment_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        e = self.assertRaises(
            exceptions.CommandError,
            self.shell,
            'deployment-list env-id --all-environments')
        self.assertIn(
            'Environment ID and all-environments flag cannot both be set.',
            e.__str__())
        self.assertFalse(self.client.deployments.list.called)

        e = self.assertRaises(
            exceptions.CommandError,
            self.shell,
            'deployment-list')
        self.assertIn(
            'Either environment ID or all-environments flag must be set.',
            e.__str__())
        self.assertFalse(self.client.deployments.list.called)

    @mock.patch('muranoclient.v1.services.ServiceManager')
    @mock.patch('muranoclient.v1.environments.EnvironmentManager')
    @requests_mock.mock()
    def test_environment_apps_edit(self, mock_env_manager, mock_services,
                                   m_requests):
        self.client.environments = mock_env_manager()
        self.client.services = mock_services()
        fake = collections.namedtuple('fakeEnv', 'services')
        self.client.environments.get.side_effect = [
            fake(services=[
                {'?': {'name': "foo"}}
            ]),
        ]

        temp_file = tempfile.NamedTemporaryFile(prefix="murano-test", mode='w')
        json.dump([
            {'op': 'replace', 'path': '/0/?/name',
                'value': "dummy"
             }
        ], temp_file)
        temp_file.file.flush()

        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)

        self.shell('environment-apps-edit 12345 {0} --session-id 4321'.format(
            temp_file.name))

        self.client.services.put.assert_called_once_with(
            '12345',
            session_id='4321',
            path='/',
            data=[{'?': {'name': 'dummy'}}]
        )

    @mock.patch('muranoclient.v1.services.ServiceManager')
    @requests_mock.mock()
    def test_app_show(self, mock_services, m_requests):
        self.client.services = mock_services()
        mock_app = mock.MagicMock()
        mock_app.name = "app_name"
        setattr(mock_app, '?', {'type': 'app_type', 'id': 'app_id'})
        self.client.services.list.return_value = [mock_app]
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        result = self.shell('app-show env-id')
        required = ['Id', 'Name', 'Type', 'app_id', 'app_name', 'app_type']
        for r in required:
            self.assertIn(r, result[0])
        self.client.services.list.assert_called_once_with('env-id')

    @mock.patch('muranoclient.v1.services.ServiceManager')
    @requests_mock.mock()
    def test_app_show_empty_list(self, mock_services, m_requests):
        self.client.services = mock_services()
        self.client.services.list.return_value = []
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        result = self.shell('app-show env-id')
        required = ['Id', 'Name', 'Type']
        for r in required:
            self.assertIn(r, result[0])
        self.client.services.list.assert_called_once_with('env-id')

    @mock.patch('muranoclient.v1.services.ServiceManager')
    @requests_mock.mock()
    def test_app_show_with_path(self, mock_services, m_requests):
        self.client.services = mock_services()
        mock_app = mock.MagicMock()
        mock_app.name = "app_name"
        setattr(mock_app, '?', {'type': 'app_type', 'id': 'app_id'})
        self.client.services.get.return_value = mock_app
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        result = self.shell('app-show env-id --path app-id')
        required = ['Property', 'Value']
        for r in required:
            self.assertIn(r, result[0])
        self.client.services.get.assert_called_once_with('env-id', '/app-id')

    @mock.patch('muranoclient.v1.categories.CategoryManager')
    @requests_mock.mock()
    def test_category_list(self, mock_manager, m_requests):
        self.client.categories = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        result = self.shell('category-list')
        required = ['ID', 'Name']
        for r in required:
            self.assertIn(r, result[0])
        self.client.categories.list.assert_called_once_with()

    @mock.patch('muranoclient.v1.packages.PackageManager')
    @mock.patch('muranoclient.v1.categories.CategoryManager')
    @requests_mock.mock()
    def test_category_show(self, category_manager, pkg_manager, m_requests):
        self.client.packages = pkg_manager()
        self.client.categories = category_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        result = self.shell('category-show category-id')
        required = ['Property', 'Value', 'id', 'name', 'packages']
        for r in required:
            self.assertIn(r, result[0])
        self.client.categories.get.assert_called_once_with('category-id')

    @mock.patch('muranoclient.v1.categories.CategoryManager')
    @requests_mock.mock()
    def test_category_create(self, mock_manager, m_requests):
        self.client.categories = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        result = self.shell('category-create category-name')
        required = ['ID', 'Name']
        for r in required:
            self.assertIn(r, result[0])
        self.client.categories.add.assert_called_once_with(
            {'name': 'category-name'})

    @mock.patch('muranoclient.v1.categories.CategoryManager')
    @requests_mock.mock()
    def test_category_delete(self, mock_manager, m_requests):
        self.client.categories = mock_manager()
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        result = self.shell('category-delete category-id')
        required = ['ID', 'Name']
        for r in required:
            self.assertIn(r, result[0])
        self.client.categories.delete.assert_called_once_with('category-id')

        self.client.categories.delete.side_effect =\
            common_exceptions.HTTPNotFound()
        ex = self.assertRaises(exceptions.CommandError, self.shell,
                               'category-delete category-id')
        expected = 'Unable to find and delete any of the specified categories.'
        self.assertEqual(expected, str(ex))


class ShellPackagesOperations(ShellCommandTest):
    @requests_mock.mock()
    def test_create_hot_based_package(self, m_requests):
        self.useFixture(fixtures.MonkeyPatch(
            'muranoclient.v1.client.Client', mock.MagicMock))
        heat_template = os.path.join(FIXTURE_DIR, 'heat-template.yaml')
        logo = os.path.join(FIXTURE_DIR, 'logo.png')
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        with tempfile.NamedTemporaryFile() as f:
            RESULT_PACKAGE = f.name
            c = "package-create --template={0} --output={1} -l={2}".format(
                heat_template, RESULT_PACKAGE, logo)
            stdout, stderr = self.shell(c)
            matchers.MatchesRegex((stdout + stderr),
                                  "Application package "
                                  "is available at {0}".format(RESULT_PACKAGE))

    @requests_mock.mock()
    def test_create_mpl_package(self, m_requests):
        self.useFixture(fixtures.MonkeyPatch(
            'muranoclient.v1.client.Client', mock.MagicMock))
        classes_dir = os.path.join(FIXTURE_DIR, 'test-app', 'Classes')
        resources_dir = os.path.join(FIXTURE_DIR, 'test-app', 'Resources')
        ui = os.path.join(FIXTURE_DIR, 'test-app', 'ui.yaml')
        self.make_env()
        self.register_keystone_discovery_fixture(m_requests)
        self.register_keystone_token_fixture(m_requests)
        with tempfile.NamedTemporaryFile() as f:
            RESULT_PACKAGE = f.name
            stdout, stderr = self.shell(
                "package-create  -c={0} -r={1} -u={2} -o={3}".format(
                    classes_dir, resources_dir, ui, RESULT_PACKAGE))
            matchers.MatchesRegex((stdout + stderr),
                                  "Application package "
                                  "is available at {0}".format(RESULT_PACKAGE))

    @mock.patch('muranoclient.common.utils.Package.from_file')
    def test_package_import(self, from_file):
        args = TestArgs()
        with tempfile.NamedTemporaryFile() as f:
            RESULT_PACKAGE = f.name
            args.filename = [RESULT_PACKAGE]
            args.categories = ['Cat1', 'Cat2 with space']
            args.is_public = True

            pkg = make_pkg({'FullName': RESULT_PACKAGE})
            from_file.return_value = utils.Package(utils.File(pkg))

            v1_shell.do_package_import(self.client, args)

            self.client.packages.create.assert_called_once_with({
                'categories': ['Cat1', 'Cat2 with space'],
                'is_public': True
            }, {RESULT_PACKAGE: mock.ANY},)

    def _test_conflict(self,
                       packages, from_file, raw_input_mock,
                       input_action, exists_action=''):
        packages.create = mock.MagicMock(
            side_effect=[common_exceptions.HTTPConflict("Conflict"), None])

        packages.filter.return_value = [mock.Mock(id='test_id')]

        raw_input_mock.return_value = input_action
        args = TestArgs()
        args.exists_action = exists_action
        with tempfile.NamedTemporaryFile() as f:
            args.filename = [f.name]

            pkg = make_pkg({'FullName': f.name})
            from_file.return_value = utils.Package(utils.File(pkg))

            v1_shell.do_package_import(self.client, args)
            return f.name

    @mock.patch('builtins.input')
    @mock.patch('muranoclient.common.utils.Package.from_file')
    def test_package_import_conflict_skip(self, from_file, raw_input_mock):

        name = self._test_conflict(
            self.client.packages,
            from_file,
            raw_input_mock,
            's',
        )

        self.client.packages.create.assert_called_once_with({
            'is_public': False,
        }, {name: mock.ANY},)

    @mock.patch('builtins.input')
    @mock.patch('muranoclient.common.utils.Package.from_file')
    def test_package_import_conflict_skip_ea(self, from_file, raw_input_mock):

        name = self._test_conflict(
            self.client.packages,
            from_file,
            raw_input_mock,
            '',
            exists_action='s',
        )

        self.client.packages.create.assert_called_once_with({
            'is_public': False,
        }, {name: mock.ANY},)
        self.assertFalse(raw_input_mock.called)

    @mock.patch('builtins.input')
    @mock.patch('muranoclient.common.utils.Package.from_file')
    def test_package_import_conflict_abort(self, from_file, raw_input_mock):

        self.assertRaises(SystemExit, self._test_conflict,
                          self.client.packages,
                          from_file,
                          raw_input_mock,
                          'a',
                          )

        self.client.packages.create.assert_called_once_with({
            'is_public': False,
        }, mock.ANY,)

    @mock.patch('builtins.input')
    @mock.patch('muranoclient.common.utils.Package.from_file')
    def test_package_import_conflict_abort_ea(self,
                                              from_file, raw_input_mock):

        self.assertRaises(SystemExit, self._test_conflict,
                          self.client.packages,
                          from_file,
                          raw_input_mock,
                          '',
                          exists_action='a',
                          )

        self.client.packages.create.assert_called_once_with({
            'is_public': False,
        }, mock.ANY,)
        self.assertFalse(raw_input_mock.called)

    @mock.patch('builtins.input')
    @mock.patch('muranoclient.common.utils.Package.from_file')
    def test_package_import_conflict_update(self, from_file, raw_input_mock):

        name = self._test_conflict(
            self.client.packages,
            from_file,
            raw_input_mock,
            'u',
        )

        self.client.packages.delete.assert_called_once_with('test_id')

        self.client.packages.create.assert_has_calls(
            [
                mock.call({'is_public': False}, {name: mock.ANY},),
                mock.call({'is_public': False}, {name: mock.ANY},)
            ], any_order=True,
        )
        self.assertEqual(2, self.client.packages.create.call_count)

    @mock.patch('builtins.input')
    @mock.patch('muranoclient.common.utils.Package.from_file')
    def test_package_import_conflict_update_ea(self,
                                               from_file, raw_input_mock):

        name = self._test_conflict(
            self.client.packages,
            from_file,
            raw_input_mock,
            '',
            exists_action='u',
        )

        self.client.packages.delete.assert_called_once_with('test_id')

        self.client.packages.create.assert_has_calls(
            [
                mock.call({'is_public': False}, {name: mock.ANY},),
                mock.call({'is_public': False}, {name: mock.ANY},)
            ], any_order=True,
        )
        self.assertEqual(2, self.client.packages.create.call_count)
        self.assertFalse(raw_input_mock.called)

    def _test_conflict_dep(self,
                           packages, from_file,
                           dep_exists_action=''):
        packages.create = mock.MagicMock(
            side_effect=[common_exceptions.HTTPConflict("Conflict"),
                         common_exceptions.HTTPConflict("Conflict"),
                         None])

        packages.filter.return_value = [mock.Mock(id='test_id')]

        args = TestArgs()
        args.exists_action = 's'
        args.dep_exists_action = dep_exists_action
        args.filename = ['first_app']

        pkg1 = make_pkg(
            {'FullName': 'first_app', 'Require': {'second_app': '1.0'}, })
        pkg2 = make_pkg({'FullName': 'second_app', })

        def side_effect(name):
            if 'first_app' in name:
                return utils.Package(utils.File(pkg1))
            if 'second_app' in name:
                return utils.Package(utils.File(pkg2))

        from_file.side_effect = side_effect

        v1_shell.do_package_import(self.client, args)

    @mock.patch('muranoclient.common.utils.Package.from_file')
    def test_package_import_conflict_dep_skip_ea(self, from_file):
        self._test_conflict_dep(
            self.client.packages,
            from_file,
            dep_exists_action='s',
        )

        self.client.packages.create.assert_has_calls(
            [
                mock.call({'is_public': False}, {'first_app': mock.ANY}),
                mock.call({'is_public': False}, {'second_app': mock.ANY}),
            ], any_order=True,
        )

        self.assertEqual(2, self.client.packages.create.call_count)

    @mock.patch('muranoclient.common.utils.Package.from_file')
    def test_package_import_conflict_dep_abort_ea(self, from_file):
        self.assertRaises(SystemExit, self._test_conflict_dep,
                          self.client.packages,
                          from_file,
                          dep_exists_action='a',
                          )

        self.client.packages.create.assert_called_with({
            'is_public': False,
        }, {'second_app': mock.ANY},)

    @mock.patch('muranoclient.common.utils.Package.from_file')
    def test_package_import_conflict_dep_update_ea(self, from_file):
        self.assertRaises(SystemExit, self._test_conflict_dep,
                          self.client.packages,
                          from_file,
                          dep_exists_action='u',
                          )

        self.assertTrue(self.client.packages.delete.called)

        self.client.packages.create.assert_has_calls(
            [
                mock.call({'is_public': False}, {'first_app': mock.ANY}),
                mock.call({'is_public': False}, {'second_app': mock.ANY}),
                mock.call({'is_public': False}, {'second_app': mock.ANY}),
            ], any_order=True,
        )

        self.assertGreater(self.client.packages.create.call_count, 2)
        self.assertLess(self.client.packages.create.call_count, 5)

    @mock.patch('muranoclient.common.utils.Package.from_file')
    def test_package_import_no_categories(self, from_file):
        args = TestArgs()

        with tempfile.NamedTemporaryFile() as f:
            RESULT_PACKAGE = f.name
            pkg = make_pkg({'FullName': RESULT_PACKAGE})
            from_file.return_value = utils.Package(utils.File(pkg))

            args.filename = [RESULT_PACKAGE]
            args.categories = None
            args.is_public = False

            v1_shell.do_package_import(self.client, args)

            self.client.packages.create.assert_called_once_with(
                {'is_public': False},
                {RESULT_PACKAGE: mock.ANY},
            )

    @requests_mock.mock()
    @mock.patch('muranoclient.common.utils.Package.from_file')
    def test_package_import_url(self, rm, from_file):
        args = TestArgs()
        args.filename = ["http://127.0.0.1/test_package.zip"]
        args.categories = None
        args.is_public = False

        pkg = make_pkg({'FullName': 'test_package'})
        from_file.return_value = utils.Package(utils.File(pkg))

        rm.get(args.filename[0], body=make_pkg({'FullName': 'test_package'}))

        v1_shell.do_package_import(self.client, args)

        self.client.packages.create.assert_called_once_with(
            {'is_public': False},
            {'test_package': mock.ANY},
        )

    @requests_mock.mock()
    @mock.patch('muranoclient.common.utils.Package.from_file')
    def test_package_import_by_name(self, rm, from_file):
        args = TestArgs()

        args.filename = ["io.test.apps.test_application"]
        args.categories = None
        args.is_public = False
        args.murano_repo_url = "http://127.0.0.1"

        pkg = make_pkg({'FullName': args.filename[0]})
        from_file.return_value = utils.Package(utils.File(pkg))

        rm.get(args.murano_repo_url + '/apps/' + args.filename[0] + '.zip',
               body=make_pkg({'FullName': 'first_app'}))

        v1_shell.do_package_import(self.client, args)

        self.assertTrue(self.client.packages.create.called)
        self.client.packages.create.assert_called_once_with(
            {'is_public': False},
            {args.filename[0]: mock.ANY},
        )

    @requests_mock.mock()
    def test_package_import_multiple(self, rm):
        args = TestArgs()

        args.filename = ["io.test.apps.test_application",
                         "http://127.0.0.1/test_app2.zip", ]
        args.categories = None
        args.is_public = False
        args.murano_repo_url = "http://127.0.0.1"

        rm.get(args.murano_repo_url + '/apps/' + args.filename[0] + '.zip',
               body=make_pkg({'FullName': 'first_app'}))

        rm.get(args.filename[1],
               body=make_pkg({'FullName': 'second_app'}))

        v1_shell.do_package_import(self.client, args)

        self.assertTrue(self.client.packages.create.called)

        self.client.packages.create.assert_has_calls(
            [
                mock.call({'is_public': False}, {'first_app': mock.ANY}),
                mock.call({'is_public': False}, {'second_app': mock.ANY}),
            ], any_order=True,
        )

    @requests_mock.mock()
    def test_import_bundle_by_name(self, m):
        """Asserts bundle import calls packages create once for each pkg."""
        pkg1 = make_pkg({'FullName': 'first_app'})
        pkg2 = make_pkg({'FullName': 'second_app'})

        m.get(TestArgs.murano_repo_url + '/apps/first_app.zip', body=pkg1)
        m.get(TestArgs.murano_repo_url + '/apps/second_app.1.0.zip',
              body=pkg2)
        s = io.StringIO()
        bundle_contents = {'Packages': [
            {'Name': 'first_app'},
            {'Name': 'second_app', 'Version': '1.0'}
        ]}
        json.dump(bundle_contents, s)
        s = io.BytesIO(s.getvalue().encode('ascii'))

        m.get(TestArgs.murano_repo_url + '/bundles/test_bundle.bundle',
              body=s)

        args = TestArgs()
        args.filename = ["test_bundle"]

        v1_shell.do_bundle_import(self.client, args)

        self.client.packages.create.assert_has_calls(
            [
                mock.call({'is_public': False}, {'first_app': mock.ANY}),
                mock.call({'is_public': False}, {'second_app': mock.ANY}),
            ], any_order=True,
        )

    @requests_mock.mock()
    def test_import_bundle_dependencies(self, m):
        """Test bundle import calls

        Asserts bundle import calls packages create once for each pkg,
        including dependencies.
        """
        pkg1 = make_pkg(
            {'FullName': 'first_app', 'Require': {'second_app': '1.0'}, })
        pkg2 = make_pkg({'FullName': 'second_app'})

        m.get(TestArgs.murano_repo_url + '/apps/first_app.zip', body=pkg1)
        m.get(TestArgs.murano_repo_url + '/apps/second_app.1.0.zip',
              body=pkg2)

        s = io.StringIO()
        # bundle only contains 1st package
        bundle_contents = {'Packages': [
            {'Name': 'first_app'},
        ]}
        json.dump(bundle_contents, s)
        s = io.BytesIO(s.getvalue().encode('ascii'))

        m.get(TestArgs.murano_repo_url + '/bundles/test_bundle.bundle',
              body=s)

        args = TestArgs()
        args.filename = ["test_bundle"]

        v1_shell.do_bundle_import(self.client, args)

        self.client.packages.create.assert_has_calls(
            [
                mock.call({'is_public': False}, {'first_app': mock.ANY}),
                mock.call({'is_public': False}, {'second_app': mock.ANY}),
            ], any_order=True,
        )

    @requests_mock.mock()
    def test_import_bundle_by_url(self, m):
        """Asserts bundle import calls packages create once for each pkg."""
        pkg1 = make_pkg({'FullName': 'first_app'})
        pkg2 = make_pkg({'FullName': 'second_app'})

        m.get(TestArgs.murano_repo_url + '/apps/first_app.zip', body=pkg1)
        m.get(TestArgs.murano_repo_url + '/apps/second_app.1.0.zip',
              body=pkg2)
        s = io.StringIO()
        bundle_contents = {'Packages': [
            {'Name': 'first_app'},
            {'Name': 'second_app', 'Version': '1.0'}
        ]}
        json.dump(bundle_contents, s)
        s = io.BytesIO(s.getvalue().encode('ascii'))

        url = 'http://127.0.0.2/test_bundle.bundle'
        m.get(url, body=s)

        args = TestArgs()
        args.filename = [url]

        v1_shell.do_bundle_import(self.client, args)

        self.client.packages.create.assert_has_calls(
            [
                mock.call({'is_public': False}, {'first_app': mock.ANY}),
                mock.call({'is_public': False}, {'second_app': mock.ANY}),
            ], any_order=True,
        )

    @requests_mock.mock()
    def test_import_bundle_wrong_url(self, m):
        url = 'http://127.0.0.2/test_bundle.bundle'
        m.get(url, status_code=404)

        args = TestArgs()
        args.filename = [url]

        v1_shell.do_bundle_import(self.client, args)
        self.assertFalse(self.client.packages.create.called)

    @requests_mock.mock()
    def test_import_bundle_no_bundle(self, m):
        url = 'http://127.0.0.1/bundles/test_bundle.bundle'
        m.get(url, status_code=404)

        args = TestArgs()
        args.filename = ["test_bundle"]

        v1_shell.do_bundle_import(self.client, args)
        self.assertFalse(self.client.packages.create.called)

    @requests_mock.mock()
    def test_import_local_bundle(self, m):
        """Asserts local bundles are first searched locally."""
        tmp_dir = tempfile.mkdtemp()
        bundle_file = os.path.join(tmp_dir, 'bundle.bundle')
        with open(os.path.join(tmp_dir, 'bundle.bundle'), 'w') as f:

            bundle_contents = {'Packages': [
                {'Name': 'first_app'},
                {'Name': 'second_app', 'Version': '1.0'}
            ]}
            json.dump(bundle_contents, f)

        pkg1 = make_pkg({'FullName': 'first_app',
                         'Require': {'third_app': None}})
        pkg2 = make_pkg({'FullName': 'second_app'})
        pkg3 = make_pkg({'FullName': 'third_app'})
        with open(os.path.join(tmp_dir, 'first_app'), 'wb') as f:
            f.write(pkg1.read())
        with open(os.path.join(tmp_dir, 'third_app'), 'wb') as f:
            f.write(pkg3.read())

        m.get(TestArgs.murano_repo_url + '/apps/first_app.zip',
              status_code=404)
        m.get(TestArgs.murano_repo_url + '/apps/second_app.1.0.zip',
              body=pkg2)
        m.get(TestArgs.murano_repo_url + '/apps/third_app.zip',
              status_code=404)

        args = TestArgs()
        args.filename = [bundle_file]
        v1_shell.do_bundle_import(self.client, args)

        self.client.packages.create.assert_has_calls(
            [
                mock.call({'is_public': False}, {'first_app': mock.ANY}),
                mock.call({'is_public': False}, {'second_app': mock.ANY}),
                mock.call({'is_public': False}, {'third_app': mock.ANY}),
            ], any_order=True,
        )
        shutil.rmtree(tmp_dir)

    @requests_mock.mock()
    def test_save_bundle(self, m):
        tmp_dir = tempfile.mkdtemp()

        pkg = make_pkg({'FullName': 'test_app'})

        expected_pkg = tempfile.NamedTemporaryFile(delete=False)
        shutil.copyfileobj(pkg, expected_pkg)
        pkg.seek(0)

        m.get(TestArgs.murano_repo_url + '/apps/test_app.zip', body=pkg)

        s = io.StringIO()
        expected_bundle = {'Packages': [
            {'Name': 'test_app'},
        ]}
        json.dump(expected_bundle, s)
        s = io.BytesIO(s.getvalue().encode('ascii'))

        m.get(TestArgs.murano_repo_url + '/bundles/test_bundle.bundle',
              body=s)

        args = TestArgs()
        args.filename = "test_bundle"
        args.path = tmp_dir

        v1_shell.do_bundle_save(self.client, args)

        expected_pkg.seek(0)
        result_bundle = json.load(open(os.path.join(
            tmp_dir, 'test_bundle.bundle')))
        result_pkg = os.path.join(tmp_dir, 'test_app.zip')

        self.assertEqual(expected_bundle, result_bundle)
        self.assertTrue(filecmp.cmp(expected_pkg.name, result_pkg))

        os.remove(expected_pkg.name)
        shutil.rmtree(tmp_dir)

    @requests_mock.mock()
    def test_package_save(self, m):
        args = TestArgs()
        tmp_dir = tempfile.mkdtemp()

        args.package = ["test_app1", "http://127.0.0.1/test_app2.zip"]
        args.path = tmp_dir

        pkgs = [
            make_pkg(
                {'FullName': 'test_app1', 'Require': {'test_app3': '1.0'}}),
            make_pkg({'FullName': 'test_app2'}),
            make_pkg({'FullName': 'test_app3'})
        ]

        m.get(TestArgs.murano_repo_url + '/apps/' + args.package[0] + '.zip',
              body=pkgs[0])
        m.get(args.package[1], body=pkgs[1])
        m.get(TestArgs.murano_repo_url + '/apps/' + 'test_app3.1.0.zip',
              body=pkgs[2])

        expected_pkgs = []

        for i in range(0, 3):
            expected_pkgs.append(tempfile.NamedTemporaryFile(delete=False))
            shutil.copyfileobj(pkgs[i], expected_pkgs[i])
            pkgs[i].seek(0)

        v1_shell.do_package_save(self.client, args)

        file_names = ['test_app1.zip', 'test_app2.zip', 'test_app3.1.0.zip']

        for i in range(0, 3):
            expected_pkgs[i].seek(0)
            result_pkg = os.path.join(tmp_dir, file_names[i])
            self.assertTrue(filecmp.cmp(expected_pkgs[i].name, result_pkg))
            os.remove(expected_pkgs[i].name)

        shutil.rmtree(tmp_dir)


class ShellPackagesOperationsV3(ShellPackagesOperations):
    def make_env(self, exclude=None, fake_env=FAKE_ENV):
        if 'OS_AUTH_URL' in fake_env:
            fake_env.update({'OS_AUTH_URL': 'http://no.where/v3'})
        env = dict((k, v) for k, v in fake_env.items() if k != exclude)
        self.useFixture(fixtures.MonkeyPatch('os.environ', env))

    def register_keystone_discovery_fixture(self, mreq):
        v3_url = "http://no.where/v3"
        v3_version = fixture.V3Discovery(v3_url)
        mreq.register_uri('GET', v3_url, json=_create_ver_list([v3_version]),
                          status_code=200)

    def register_keystone_token_fixture(self, mreq):
        v3_token = ks_v3_fixture.Token()
        service = v3_token.add_service('application-catalog')
        service.add_standard_endpoints(public='http://no.where')
        mreq.register_uri('POST',
                          'http://no.where/v3/auth/tokens',
                          json=v3_token,
                          headers={'X-Subject-Token': 'tokenid'},
                          status_code=200)



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from muranoclient.common import http
from muranoclient.v1 import actions
from muranoclient.v1 import artifact_packages
from muranoclient.v1 import categories
from muranoclient.v1 import deployments
from muranoclient.v1 import environments
from muranoclient.v1 import instance_statistics
from muranoclient.v1 import packages
from muranoclient.v1 import request_statistics
from muranoclient.v1 import schemas
from muranoclient.v1 import services
from muranoclient.v1 import sessions
from muranoclient.v1 import static_actions
from muranoclient.v1 import templates


class Client(object):
    """Client for the Murano v1 API.

    :param string endpoint: A user-supplied endpoint URL for the service.
    :param string token: Token for authentication.
    :param integer timeout: Allows customization of the timeout for client
                            http requests. (optional)
    """

    def __init__(self, *args, **kwargs):
        """Initialize a new client for the Murano v1 API."""
        self.glance_client = kwargs.pop('glance_client', None)
        tenant = kwargs.pop('tenant', None)
        artifacts_client = kwargs.pop('artifacts_client', None)
        self.http_client = http._construct_http_client(*args, **kwargs)
        self.environments = environments.EnvironmentManager(self.http_client)
        self.env_templates = templates.EnvTemplateManager(self.http_client)
        self.sessions = sessions.SessionManager(self.http_client)
        self.services = services.ServiceManager(self.http_client)
        self.deployments = deployments.DeploymentManager(self.http_client)
        self.schemas = schemas.SchemaManager(self.http_client)
        self.request_statistics = \
            request_statistics.RequestStatisticsManager(self.http_client)
        self.instance_statistics = \
            instance_statistics.InstanceStatisticsManager(self.http_client)
        pkg_mgr = packages.PackageManager(self.http_client)
        if artifacts_client:
            artifact_repo = artifact_packages.ArtifactRepo(artifacts_client,
                                                           tenant)
            self.packages = artifact_packages.PackageManagerAdapter(
                pkg_mgr, artifact_repo)
        else:
            self.packages = pkg_mgr
        self.actions = actions.ActionManager(self.http_client)
        self.static_actions = static_actions.StaticActionManager(
            self.http_client)
        self.categories = categories.CategoryManager(self.http_client)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from unittest import mock

from muranoclient.common import exceptions as exc

HTML_MSG = """<html>
 <head>
  <title>403 Forbidden</title>
 </head>
 <body>
  <h1>403 Forbidden</h1>
  Access was denied to this resource.<br /><br />
 </body>
</html>"""


class TestHTTPExceptions(testtools.TestCase):
    def test_handles_json(self):
        """exc.from_response should not print JSON."""
        mock_resp = mock.Mock()
        mock_resp.status_code = 413
        mock_resp.json.return_value = {
            "overLimit": {
                "code": 413,
                "message": "OverLimit Retry...",
                "details": "Error Details...",
                "retryAt": "2015-08-31T21:21:06Z"
            }
        }
        mock_resp.headers = {
            "content-type": "application/json"
        }
        err = exc.from_response(mock_resp)
        self.assertIsInstance(err, exc.HTTPOverLimit)
        self.assertEqual("OverLimit Retry...", err.details)

    def test_handles_html(self):
        """exc.from_response should not print HTML."""
        mock_resp = mock.Mock()
        mock_resp.status_code = 403
        mock_resp.text = HTML_MSG
        mock_resp.headers = {
            "content-type": "text/html"
        }
        err = exc.from_response(mock_resp)
        self.assertIsInstance(err, exc.HTTPForbidden)
        self.assertEqual("403 Forbidden: Access was denied to this resource.",
                         err.details)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Application-catalog v1 action implementation"""

import json

from osc_lib.command import command
from oslo_log import log as logging

from muranoclient.apiclient import exceptions

LOG = logging.getLogger(__name__)


class StaticActionCall(command.ShowOne):
    """Call static method of the MuranoPL class."""

    def get_parser(self, prog_name):
        parser = super(StaticActionCall, self).get_parser(prog_name)
        parser.add_argument(
            "class_name",
            metavar='<CLASS>',
            help="FQN of the class with static method",
        )
        parser.add_argument(
            "method_name",
            metavar='<METHOD>',
            help="Static method to run",
        )
        parser.add_argument(
            "--arguments",
            metavar='<KEY=VALUE>',
            nargs='*',
            help="Method arguments. No arguments by default",
        )
        parser.add_argument(
            "--package-name",
            metavar='<PACKAGE>',
            default='',
            help='Optional FQN of the package to look for the class in',
        )
        parser.add_argument(
            "--class-version",
            default='',
            help='Optional version of the class, otherwise version =0 is '
                 'used ',
        )

        return parser

    def take_action(self, parsed_args):
        LOG.debug("take_action({0})".format(parsed_args))
        client = self.app.client_manager.application_catalog

        arguments = {}
        for argument in parsed_args.arguments or []:
            if '=' not in argument:
                raise exceptions.CommandError(
                    "Argument should be in form of KEY=VALUE. Found: "
                    "{0}".format(argument))
            key, value = argument.split('=', 1)
            try:
                value = json.loads(value)
            except ValueError:
                # treat value as a string if it doesn't load as json
                pass

            arguments[key] = value

        request_body = {
            "className": parsed_args.class_name,
            "methodName": parsed_args.method_name,
            "packageName": parsed_args.package_name or None,
            "classVersion": parsed_args.class_version or '=0',
            "parameters": arguments
        }

        print("Waiting for result...")
        result = client.static_actions.call(request_body).get_result()
        return ["Static action result"], [result]



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from unittest import mock

from muranoclient import client
from muranoclient.v1 import actions
from muranoclient.v1 import deployments
import muranoclient.v1.environments as environments
from muranoclient.v1 import packages
import muranoclient.v1.sessions as sessions
from muranoclient.v1 import static_actions
import muranoclient.v1.templates as templates


def my_mock(*a, **b):
    return [a, b]


api = mock.MagicMock(json_request=my_mock)


class UnitTestsForClassesAndFunctions(testtools.TestCase):
    def test_create_client_instance(self):

        endpoint = 'http://no-resolved-host:8001'
        test_client = client.Client('1', endpoint=endpoint,
                                    token='1', timeout=10)

        self.assertIsNotNone(test_client.environments)
        self.assertIsNotNone(test_client.sessions)
        self.assertIsNotNone(test_client.services)

    def test_env_manager_list(self):
        manager = environments.EnvironmentManager(api)
        result = manager.list()

        self.assertEqual([], result)

    def test_env_manager_create(self):
        manager = environments.EnvironmentManager(api)
        result = manager.create({'name': 'test'})

        self.assertEqual({'name': 'test'}, result.data)

    def test_env_manager_create_with_named_parameters(self):
        manager = environments.EnvironmentManager(api)
        result = manager.create(data={'name': 'test'})

        self.assertEqual({'name': 'test'}, result.data)

    def test_env_manager_create_negative_without_parameters(self):

        manager = environments.EnvironmentManager(api)

        self.assertRaises(TypeError, manager.create)

    def test_env_manager_delete(self):
        manager = environments.EnvironmentManager(api)
        result = manager.delete('test')

        self.assertIsNone(result)

    def test_env_manager_delete_with_named_parameters(self):
        manager = environments.EnvironmentManager(api)
        result = manager.delete(environment_id='1')

        self.assertIsNone(result)

    def test_env_manager_delete_negative_without_parameters(self):

        manager = environments.EnvironmentManager(api)

        self.assertRaises(TypeError, manager.delete)

    def test_env_manager_update(self):
        manager = environments.EnvironmentManager(api)
        result = manager.update('1', 'test')

        self.assertEqual({'name': 'test'}, result.data)

    def test_env_manager_update_with_named_parameters(self):
        manager = environments.EnvironmentManager(api)
        result = manager.update(environment_id='1',
                                name='test')

        self.assertEqual({'name': 'test'}, result.data)

    def test_env_manager_update_negative_with_one_parameter(self):

        manager = environments.EnvironmentManager(api)

        self.assertRaises(TypeError, manager.update, 'test')

    def test_env_manager_update_negative_without_parameters(self):

        manager = environments.EnvironmentManager(api)

        self.assertRaises(TypeError, manager.update)

    def test_env_manager_get(self):
        manager = environments.EnvironmentManager(api)
        result = manager.get('test')

        self.assertIsNotNone(result.manager)

    def test_env(self):
        environment = environments.Environment(api, api)

        self.assertIsNotNone(environment.data())

    def test_session_manager_delete(self):
        manager = sessions.SessionManager(api)
        result = manager.delete('datacenter1', 'session1')

        self.assertIsNone(result)

    def test_session_manager_delete_with_named_parameters(self):
        manager = sessions.SessionManager(api)
        result = manager.delete(environment_id='datacenter1',
                                session_id='session1')

        self.assertIsNone(result)

    def test_session_manager_delete_negative_with_one_parameter(self):

        manager = sessions.SessionManager(api)

        self.assertRaises(TypeError, manager.delete, 'datacenter1')

    def test_session_manager_delete_negative_without_parameters(self):

        manager = sessions.SessionManager(api)

        self.assertRaises(TypeError, manager.delete)

    def test_session_manager_get(self):
        manager = sessions.SessionManager(api)
        result = manager.get('datacenter1', 'session1')
        # WTF?
        self.assertIsNotNone(result.manager)

    def test_session_manager_configure(self):
        manager = sessions.SessionManager(api)
        result = manager.configure('datacenter1')

        self.assertIsNotNone(result)

    def test_session_manager_configure_with_named_parameter(self):
        manager = sessions.SessionManager(api)
        result = manager.configure(environment_id='datacenter1')

        self.assertIsNotNone(result)

    def test_session_manager_configure_negative_without_parameters(self):

        manager = sessions.SessionManager(api)

        self.assertRaises(TypeError, manager.configure)

    def test_session_manager_deploy(self):
        manager = sessions.SessionManager(api)
        result = manager.deploy('datacenter1', '1')

        self.assertIsNone(result)

    def test_session_manager_deploy_with_named_parameters(self):
        manager = sessions.SessionManager(api)
        result = manager.deploy(environment_id='datacenter1',
                                session_id='1')

        self.assertIsNone(result)

    def test_session_manager_deploy_negative_with_one_parameter(self):

        manager = sessions.SessionManager(api)

        self.assertRaises(TypeError, manager.deploy, 'datacenter1')

    def test_session_manager_deploy_negative_without_parameters(self):

        manager = sessions.SessionManager(api)

        self.assertRaises(TypeError, manager.deploy)

    def test_action_manager_call(self):
        api_mock = mock.MagicMock(
            json_request=lambda *args, **kwargs: (None, {'task_id': '1234'}))
        manager = actions.ActionManager(api_mock)
        result = manager.call('testEnvId', 'testActionId', ['arg1', 'arg2'])
        self.assertEqual('1234', result)

    def test_package_filter_pagination_next_marker(self):
        # ``PackageManager.filter`` handles `next_marker` parameter related
        # to pagination in API correctly.
        responses = [
            {'next_marker': 'test_marker',
             'packages': [{'name': 'test_package_1'}]},
            {'packages': [{'name': 'test_package_2'}]}
        ]

        def json_request(url, method, *args, **kwargs):
            self.assertIn('/v1/catalog/packages', url)

            return mock.MagicMock(), responses.pop(0)

        api = mock.MagicMock()
        api.configure_mock(**{'json_request.side_effect': json_request})

        manager = packages.PackageManager(api)
        list(manager.filter())

        self.assertEqual(2, api.json_request.call_count)

    def test_package_filter_encoding_good(self):
        responses = [
            {'next_marker': 'test_marker',
             'packages': [{'name': 'test_package_1'}]},
            {'packages': [{'name': 'test_package_2'}]}
        ]

        def json_request(url, method, *args, **kwargs):
            self.assertIn('category=%D0%BF%D0%B8%D0%B2%D0%BE', url)
            return mock.MagicMock(), responses.pop(0)

        api = mock.MagicMock()
        api.configure_mock(**{'json_request.side_effect': json_request})

        manager = packages.PackageManager(api)
        category = b'\xd0\xbf\xd0\xb8\xd0\xb2\xd0\xbe'
        kwargs = {'category': category.decode('utf-8')}
        list(manager.filter(**kwargs))

        self.assertEqual(2, api.json_request.call_count)

    def test_action_manager_get_result(self):
        api_mock = mock.MagicMock(
            json_request=lambda *args, **kwargs: (None, {'a': 'b'}))
        manager = actions.ActionManager(api_mock)
        result = manager.get_result('testEnvId', '1234')
        self.assertEqual({'a': 'b'}, result)

    def test_static_action_manager_call(self):
        api_mock = mock.MagicMock(
            json_request=lambda *args, **kwargs: (None, 'result'))
        manager = static_actions.StaticActionManager(api_mock)
        args = {'className': 'cls', 'methodName': 'method'}
        result = manager.call(args).get_result()
        self.assertEqual('result', result)

    def test_env_template_manager_list(self):
        """Tests the list of environment templates."""
        manager = templates.EnvTemplateManager(api)
        result = manager.list()

        self.assertEqual([], result)

    def test_env_template_manager_create(self):
        manager = templates.EnvTemplateManager(api)
        result = manager.create({'name': 'test'})

        self.assertEqual({'name': 'test'}, result.data)

    def test_env_template_manager_create_with_named_parameters(self):
        manager = templates.EnvTemplateManager(api)
        result = manager.create(data={'name': 'test'})

        self.assertEqual({'name': 'test'}, result.data)

    def test_env_template_manager_create_negative_without_parameters(self):
        manager = templates.EnvTemplateManager(api)
        self.assertRaises(TypeError, manager.create)

    def test_env_template_manager_delete(self):
        manager = templates.EnvTemplateManager(api)
        result = manager.delete('test')

        self.assertIsNone(result)

    def test_env_template_manager_delete_with_named_parameters(self):
        manager = templates.EnvTemplateManager(api)
        result = manager.delete(env_template_id='1')

        self.assertIsNone(result)

    def test_env_template_manager_delete_negative_without_parameters(self):

        manager = templates.EnvTemplateManager(api)

        self.assertRaises(TypeError, manager.delete)

    def test_env_template_manager_update(self):
        manager = templates.EnvTemplateManager(api)
        result = manager.update('1', 'test')

        self.assertEqual({'name': 'test'}, result.data)

    def test_env_template_manager_update_with_named_parameters(self):
        manager = templates.EnvTemplateManager(api)
        result = manager.update(env_template_id='1',
                                name='test')

        self.assertEqual({'name': 'test'}, result.data)

    def test_env_template_manager_update_negative_with_one_parameter(self):

        manager = templates.EnvTemplateManager(api)

        self.assertRaises(TypeError, manager.update, 'test')

    def test_env_template_manager_update_negative_without_parameters(self):

        manager = templates.EnvTemplateManager(api)

        self.assertRaises(TypeError, manager.update)

    def test_env_template_manager_get(self):
        manager = templates.EnvTemplateManager(api)
        result = manager.get('test')

        self.assertIsNotNone(result.manager)

    def test_deployment_manager_list(self):
        manager = deployments.DeploymentManager(api)
        manager._list = mock.Mock(return_value=mock.sentinel.deployments)

        result = manager.list(mock.sentinel.environment_id)
        self.assertEqual(mock.sentinel.deployments, result)

        manager._list.assert_called_once_with(
            '/v1/environments/sentinel.environment_id/deployments',
            'deployments')

    def test_deployment_manager_list_all_environments(self):
        manager = deployments.DeploymentManager(api)
        manager._list = mock.Mock(return_value=mock.sentinel.deployments)

        result = manager.list(mock.sentinel.environment_id,
                              all_environments=True)
        self.assertEqual(mock.sentinel.deployments, result)

        manager._list.assert_called_once_with('/v1/deployments', 'deployments')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""OpenStackClient plugin for Application Catalog service."""

from osc_lib import utils
from oslo_log import log as logging

from muranoclient.apiclient import exceptions as exc
from muranoclient.glance import client as art_client
from muranoclient.i18n import _

LOG = logging.getLogger(__name__)

DEFAULT_APPLICATION_CATALOG_API_VERSION = "1"
API_VERSION_OPTION = "os_application_catalog_api_version"
API_NAME = "application_catalog"
API_VERSIONS = {
    '1': 'muranoclient.v1.client.Client',
}


def make_client(instance):
    """Returns an application-catalog service client"""
    application_catalog_client = utils.get_client_class(
        API_NAME,
        instance._api_version[API_NAME],
        API_VERSIONS)
    LOG.debug("Instantiating application-catalog client: {0}".format(
              application_catalog_client))

    kwargs = {
        'session': instance.session,
        'service_type': 'application-catalog',
        'region_name': instance._region_name
    }

    murano_packages_service = \
        instance.get_configuration().get('murano_packages_service')

    if murano_packages_service == 'glare':
        glare_endpoint = instance.get_configuration().get('glare_url')
        if not glare_endpoint:
            try:
                # no glare_endpoint and we requested to store packages in glare
                # check keystone catalog
                glare_endpoint = instance.get_endpoint_for_service_type(
                    'artifact',
                    region_name=instance._region_name,
                    interface=instance._interface
                )
            except Exception:
                raise exc.CommandError(
                    "You set murano-packages-service to {}"
                    " but there is not 'artifact' endpoint in keystone"
                    " Either register one or specify endpoint "
                    " via either --glare-url or env[GLARE_API]".format(
                        murano_packages_service))

        artifacts_client = art_client.Client(
            endpoint=glare_endpoint,
            type_name='murano',
            type_version=1,
            token=instance.auth_ref.auth_token)
        kwargs['artifacts_client'] = artifacts_client

    murano_endpoint = instance.get_configuration().get('murano_url')
    if not murano_endpoint:
        murano_endpoint = instance.get_endpoint_for_service_type(
            'application-catalog',
            region_name=instance._region_name,
            interface=instance._interface
        )

    client = application_catalog_client(murano_endpoint, **kwargs)
    return client


def build_option_parser(parser):
    """Hook to add global options"""
    parser.add_argument(
        '--os-application-catalog-api-version',
        metavar='<application-catalog-api-version>',
        default=utils.env(
            'OS_APPLICATION_CATALOG_API_VERSION',
            default=DEFAULT_APPLICATION_CATALOG_API_VERSION),
        help=_("Application catalog API version, default={0}"
               "(Env:OS_APPLICATION_CATALOG_API_VERSION)").format(
                   DEFAULT_APPLICATION_CATALOG_API_VERSION))
    parser.add_argument('--murano-url',
                        default=utils.env('MURANO_URL'),
                        help=_('Defaults to env[MURANO_URL].'))
    parser.add_argument('--glare-url',
                        default=utils.env('GLARE_URL'),
                        help='Defaults to env[GLARE_URL].')
    parser.add_argument('--murano-packages-service',
                        choices=['murano', 'glare'],
                        default=utils.env('MURANO_PACKAGES_SERVICE',
                                          default='murano'),
                        help='Specifies if murano-api ("murano") or '
                             'Glance Artifact Repository ("glare") '
                             'should be used to store murano packages. '
                             'Defaults to env[MURANO_PACKAGES_SERVICE] or '
                             'to "murano"')
    return parser



# Copyright (c) 2015 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from glanceclient.common import utils
from glanceclient import exc
from oslo_utils import encodeutils
from urllib import parse

from muranoclient.glance import ArtifactType


glare_urls = {
    'create': '/v{version}/artifacts/{type_name}/v{type_version}/drafts',
    'update_get_delete': '/v{version}/artifacts/{type_name}/v{type_version}'
                         '/{artifact_id}',
    'list_drafts': '/v{version}/artifacts/{type_name}/v{type_version}/drafts?',
    'list_no_drafts': '/v{version}/artifacts/{type_name}/v{type_version}?',
    'publish': '/v{version}/artifacts/{type_name}/v{type_version}/'
               '{artifact_id}/publish',
    'blob': '/v{version}/artifacts/{type_name}/v{type_version}/{artifact_id}'
            '/{blob_property}',
}


class Controller(object):
    def __init__(self, http_client, type_name=None, type_version=None,
                 version='0.1'):
        self.http_client = http_client
        self.type_name = type_name
        self.type_version = type_version
        self.version = version
        self.default_page_size = 20
        self.sort_dir_values = ('asc', 'desc')

    def _check_type_params(self, type_name, type_version):
        """Check that type name and type versions were specified"""
        type_name = type_name or self.type_name
        type_version = type_version or self.type_version

        if type_name is None:
            msg = "Type name must be specified"
            raise exc.HTTPBadRequest(msg)

        if type_version is None:
            msg = "Type version must be specified"
            raise exc.HTTPBadRequest(msg)

        return type_name, type_version

    def _validate_sort_param(self, sort):
        """Validates sorting argument for invalid keys and directions values.

        :param sort: comma-separated list of sort keys with optional <:dir>
        after each key
        """
        for sort_param in sort.strip().split(','):
            key, _sep, dir = sort_param.partition(':')
            if dir and dir not in self.sort_dir_values:
                msg = ('Invalid sort direction: %(sort_dir)s.'
                       ' It must be one of the following: %(available)s.'
                       ) % {'sort_dir': dir,
                            'available': ', '.join(self.sort_dir_values)}
                raise exc.HTTPBadRequest(msg)
        return sort

    def create(self, name, version, type_name=None, type_version=None,
               **kwargs):
        """Create an artifact of given type and version.

        :param name: name of creating artifact.
        :param version: semver string describing an artifact version
        """
        type_name, type_version = self._check_type_params(type_name,
                                                          type_version)
        kwargs.update({'name': name, 'version': version})
        url = glare_urls['create'].format(version=self.version,
                                          type_name=type_name,
                                          type_version=type_version)
        resp, body = self.http_client.post(url, data=kwargs)
        return ArtifactType(**body)

    def update(self, artifact_id, type_name=None, type_version=None,
               remove_props=None, **kwargs):
        """Update attributes of an artifact.

        :param artifact_id: ID of the artifact to modify.
        :param remove_props: List of property names to remove
        :param **kwargs: Artifact attribute names and their new values.
        """
        type_name, type_version = self._check_type_params(type_name,
                                                          type_version)
        url = glare_urls['update_get_delete'].format(version=self.version,
                                                     type_name=type_name,
                                                     type_version=type_version,
                                                     artifact_id=artifact_id)
        hdrs = {
            'Content-Type': 'application/openstack-images-v2.1-json-patch'}

        artifact_obj = self.get(artifact_id, type_name, type_version)

        changes = []
        if remove_props:
            for prop in remove_props:
                if prop in ArtifactType.generic_properties:
                    msg = "Generic properties cannot be removed"
                    raise exc.HTTPBadRequest(msg)
                if prop not in kwargs:
                    changes.append({'op': 'remove',
                                    'path': '/' + prop})

        for prop in kwargs:
            if prop in artifact_obj.generic_properties:
                op = 'add' if getattr(artifact_obj,
                                      prop) is None else 'replace'
            elif prop in artifact_obj.type_specific_properties:
                if artifact_obj.type_specific_properties[prop] is None:
                    op = 'add'
                else:
                    op = 'replace'
            else:
                msg = ("Property '%s' doesn't exist in type '%s' with version"
                       " '%s'" % (prop, type_name, type_version))
                raise exc.HTTPBadRequest(msg)
            changes.append({'op': op, 'path': '/' + prop,
                            'value': kwargs[prop]})

        resp, body = self.http_client.patch(url, headers=hdrs, data=changes)
        return ArtifactType(**body)

    def get(self, artifact_id, type_name=None, type_version=None,
            show_level=None):
        """Get information about an artifact.

        :param artifact_id: ID of the artifact to get.
        :param show_level: value of datalization. Possible values:
                           "none", "basic", "direct", "transitive"
        """
        type_name, type_version = self._check_type_params(type_name,
                                                          type_version)

        url = glare_urls['update_get_delete'].format(version=self.version,
                                                     type_name=type_name,
                                                     type_version=type_version,
                                                     artifact_id=artifact_id)
        if show_level:
            if show_level not in ArtifactType.supported_show_levels:
                msg = "Invalid show level: %s" % show_level
                raise exc.HTTPBadRequest(msg)
            url += '?show_level=%s' % show_level
        resp, body = self.http_client.get(url)
        return ArtifactType(**body)

    def list(self, **kwargs):
        return self._list(drafts=False, **kwargs)

    def drafts(self, **kwargs):
        return self._list(drafts=True, **kwargs)

    def _list(self, drafts, type_name=None, type_version=None, **kwargs):
        """Retrieve a listing of Image objects.

        :param page_size: Number of images to request in each
                          paginated request.
        :returns: generator over list of artifacts.
        """
        type_name, type_version = self._check_type_params(type_name,
                                                          type_version)

        limit = kwargs.get('limit')
        page_size = kwargs.get('page_size') or self.default_page_size

        def paginate(url, page_size, limit=None):
            next_url = url

            while True:
                if limit and page_size > limit:
                    next_url = next_url.replace("limit=%s" % page_size,
                                                "limit=%s" % limit)

                resp, body = self.http_client.get(next_url)
                for artifact in body['artifacts']:
                    yield ArtifactType(**artifact)

                    if limit:
                        limit -= 1
                        if limit <= 0:
                            raise StopIteration

                try:
                    next_url = body['next']
                except KeyError:
                    return

        filters = kwargs.get('filters', {})
        filters['limit'] = page_size

        url_params = []
        for param, items in filters.items():
            values = [items] if not isinstance(items, list) else items
            for value in values:
                if isinstance(value, str):
                    value = encodeutils.safe_encode(value)
                url_params.append({param: value})

        if drafts:
            url = glare_urls['list_drafts'].format(version=self.version,
                                                   type_name=type_name,
                                                   type_version=type_version)
        else:
            url = glare_urls['list_no_drafts'].format(
                version=self.version,
                type_name=type_name,
                type_version=type_version
            )

        for param in url_params:
            url = '%s&%s' % (url, parse.urlencode(param))

        if 'sort' in kwargs:
            url = '%s&sort=%s' % (url, self._validate_sort_param(
                kwargs['sort']))

        for artifact in paginate(url, page_size, limit):
            yield artifact

    def active(self, artifact_id, type_name=None, type_version=None):
        """Set artifact status to 'active'.

        :param artifact_id: ID of the artifact to get.
        """
        type_name, type_version = self._check_type_params(type_name,
                                                          type_version)

        url = glare_urls['publish'].format(version=self.version,
                                           type_name=type_name,
                                           type_version=type_version,
                                           artifact_id=artifact_id)

        resp, body = self.http_client.post(url)
        return ArtifactType(**body)

    def deactivate(self, artifact_id, type_name=None, type_version=None):
        raise NotImplementedError()

    def delete(self, artifact_id, type_name=None, type_version=None):
        """Delete an artifact and all its data.

        :param artifact_id: ID of the artifact to delete.
        """
        type_name, type_version = self._check_type_params(type_name,
                                                          type_version)
        url = glare_urls['update_get_delete'].format(version=self.version,
                                                     type_name=type_name,
                                                     type_version=type_version,
                                                     artifact_id=artifact_id)
        self.http_client.delete(url)

    def upload_blob(self, artifact_id, blob_property, data, position=None,
                    type_name=None, type_version=None):
        """Upload blob data.

        :param artifact_id: ID of the artifact to download a blob
        :param blob_property: blob property name
        :param position: if blob_property is a list then the
        position must be specified
        """
        type_name, type_version = self._check_type_params(type_name,
                                                          type_version)
        hdrs = {'Content-Type': 'application/octet-stream'}

        url = glare_urls['blob'].format(version=self.version,
                                        type_name=type_name,
                                        type_version=type_version,
                                        artifact_id=artifact_id,
                                        blob_property=blob_property)
        if position:
            url += "/%s" % position

        self.http_client.put(url, headers=hdrs, data=data)

    def download_blob(self, artifact_id, blob_property, position=None,
                      type_name=None, type_version=None, do_checksum=True):
        """Get blob data.

        :param artifact_id: ID of the artifact to download a blob
        :param blob_property: blob property name
        :param position: if blob_property is a list then the
        position must be specified
        :param do_checksum: Enable/disable checksum validation.
        """
        type_name, type_version = self._check_type_params(type_name,
                                                          type_version)
        url = glare_urls['blob'].format(version=self.version,
                                        type_name=type_name,
                                        type_version=type_version,
                                        artifact_id=artifact_id,
                                        blob_property=blob_property)
        if position:
            url += '/%s' % position

        url += '/download'

        resp, body = self.http_client.get(url)
        checksum = resp.headers.get('content-md5', None)
        content_length = int(resp.headers.get('content-length', 0))

        if checksum is not None and do_checksum:
            body = utils.integrity_iter(body, checksum)

        return utils.IterableWithLength(body, content_length)

    def delete_blob(self, artifact_id, blob_property, position=None,
                    type_name=None, type_version=None):
        """Delete blob and related data.

        :param artifact_id: ID of the artifact to delete a blob
        :param blob_property: blob property name
        :param position: if blob_property is a list then the
        position must be specified
        """
        type_name, type_version = self._check_type_params(type_name,
                                                          type_version)
        url = glare_urls['blob'].format(version=self.version,
                                        type_name=type_name,
                                        type_version=type_version,
                                        artifact_id=artifact_id,
                                        blob_property=blob_property)
        if position:
            url += '/%s' % position
        self.http_client.delete(url)

    def add_property(self, artifact_id, dependency_id, position=None,
                     type_name=None, type_version=None):
        raise NotImplementedError()

    def replace_property(self, artifact_id, dependency_id, position=None,
                         type_name=None, type_version=None):
        raise NotImplementedError()

    def remove_property(self, artifact_id, dependency_id, position=None,
                        type_name=None, type_version=None):
        raise NotImplementedError()

    def artifact_export(self, artifact_id,
                        type_name=None, type_version=None):
        raise NotImplementedError()

    def artifact_import(self, data, type_name=None, type_version=None):
        raise NotImplementedError()



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from muranoclient.common import base


class Session(base.Resource):
    def __repr__(self):
        return '<Session %s>' % self._info

    def data(self, **kwargs):
        return self.manager.data(self, **kwargs)


class SessionManager(base.Manager):
    resource_class = Session

    def get(self, environment_id, session_id):
        return self._get('/v1/environments/{id}/sessions/{session_id}'.
                         format(id=environment_id, session_id=session_id))

    def configure(self, environment_id):
        return self._create('/v1/environments/{id}/configure'.
                            format(id=environment_id), None)

    def deploy(self, environment_id, session_id):
        path = '/v1/environments/{id}/sessions/{session_id}/deploy'
        self.api.json_request(path.format(id=environment_id,
                                          session_id=session_id), 'POST')

    def delete(self, environment_id, session_id):
        return self._delete("/v1/environments/{id}/sessions/{session_id}".
                            format(id=environment_id, session_id=session_id))



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from muranoclient.common import base


class Template(base.Resource):
    """Involves the template resource."""
    def __repr__(self):
        return "<Template %s>" % self._info

    def data(self, **kwargs):
        return self.manager.data(self, **kwargs)


class EnvTemplateManager(base.Manager):
    """Involves the template manager."""
    resource_class = Template

    def list(self):
        """Lists the environment templates."""
        return self._list('/v1/templates', 'templates')

    def create(self, data):
        """Creates a environment template

        :param data: The environment template information.
        """
        return self._create('/v1/templates', data)

    def update(self, env_template_id, name):
        """Updates the environment template name.

        :param env_template_id: The environment template ID.
        :param name: The name to be updated.
        """
        return self._update('/v1/templates/{id}'.format(id=env_template_id),
                            data={'name': name})

    def delete(self, env_template_id):
        """Deletes an environment template name.

        :param env_template_id: The environment template ID.
        """
        return self._delete('/v1/templates/{id}'.format(id=env_template_id))

    def get(self, env_template_id):
        """Gets information about an environment template name.

        :param env_template_id: The environment template ID.
        """
        return self._get("/v1/templates/{id}".format(id=env_template_id))

    def create_app(self, env_template_id, data):
        """Creates an application in an environment template.

        :param env_template_id: The environment template ID.
        :param data: the application information.
        """
        return self.\
            _create('/v1/templates/{id}/services'.
                    format(id=env_template_id), data)

    def delete_app(self, env_template_id, app_id):
        """Deletes an application in an environment template.

        :param env_template_id: The environment template ID.
        :param app_id: the application ID to be deleted.
        """
        return self._delete('/v1/templates/{id}/services/{app_id}'.
                            format(id=env_template_id, app_id=app_id))

    def create_env(self, env_template_id, data):
        """Creates new environment from template.

        :param env_template_id: The environment template ID.
        :param data: The environment information.
        """
        return self._create('/v1/templates/{id}/create-environment'.
                            format(id=env_template_id), data=data)

    def clone(self, env_template_id, name):
        """Clones a public template from one tenant to another.

        :param env_template_id: The environment template ID to be cloned.
        :param name: The name for the new template.
        """
        return self._create('/v1/templates/{id}/clone'.
                            format(id=env_template_id), data={'name': name})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Application-catalog v1 category action implementation"""

import textwrap

from osc_lib.command import command
from osc_lib import utils
from oslo_log import log as logging

from muranoclient.apiclient import exceptions

LOG = logging.getLogger(__name__)


class ListCategories(command.Lister):
    """List all available categories."""

    def take_action(self, parsed_args=None):
        LOG.debug("take_action({0})".format(parsed_args))
        client = self.app.client_manager.application_catalog

        if parsed_args is None:
            parsed_args = {}

        data = client.categories.list()

        fields = ["id", "name"]
        field_labels = ["ID", "Name"]

        return (
            field_labels,
            list(utils.get_item_properties(
                s,
                fields,
            ) for s in data)
        )


class ShowCategory(command.ShowOne):
    """Display category details."""

    def get_parser(self, prog_name):
        parser = super(ShowCategory, self).get_parser(prog_name)
        parser.add_argument(
            "id",
            metavar="<ID>",
            help=("ID of a category(s) to show."),
        )

        return parser

    def take_action(self, parsed_args):
        LOG.debug("take_action({0})".format(parsed_args))
        client = self.app.client_manager.application_catalog

        category = client.categories.get(parsed_args.id)
        packages = client.packages.filter(category=category.name)
        to_display = dict(id=category.id,
                          name=category.name,
                          packages=', '.join(p.name
                                             for p in packages))

        to_display['packages'] = '\n'.join(textwrap.wrap(to_display['packages']
                                           or '', 55))

        return self.dict2columns(to_display)


class CreateCategory(command.Lister):
    """Create a category."""

    def get_parser(self, prog_name):
        parser = super(CreateCategory, self).get_parser(prog_name)
        parser.add_argument(
            "name",
            metavar="<CATEGORY_NAME>",
            help=("Category name."),
        )

        return parser

    def take_action(self, parsed_args):
        LOG.debug("take_action({0})".format(parsed_args))
        client = self.app.client_manager.application_catalog

        data = [client.categories.add({"name": parsed_args.name})]

        fields = ["id", "name"]
        field_labels = ["ID", "Name"]

        return (
            field_labels,
            list(utils.get_item_properties(
                s,
                fields,
            ) for s in data)
        )


class DeleteCategory(command.Lister):
    """Delete a category."""

    def get_parser(self, prog_name):
        parser = super(DeleteCategory, self).get_parser(prog_name)
        parser.add_argument(
            "id",
            metavar="<ID>",
            nargs="+",
            help=("ID of a category(ies) to delete."),
        )

        return parser

    def take_action(self, parsed_args):
        LOG.debug("take_action({0})".format(parsed_args))
        client = self.app.client_manager.application_catalog

        failure_count = 0
        for category_id in parsed_args.id:
            try:
                client.categories.delete(category_id)
            except Exception:
                failure_count += 1
                print("Failed to delete '{0}'; category not found".
                      format(category_id))
        if failure_count == len(parsed_args.id):
            raise exceptions.CommandError("Unable to find and delete any of "
                                          "the specified categories.")
        data = client.categories.list()

        fields = ["id", "name"]
        field_labels = ["ID", "Name"]

        return (
            field_labels,
            list(utils.get_item_properties(
                s,
                fields,
            ) for s in data)
        )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from muranoclient.osc.v1 import schema as osc_schema
from muranoclient.tests.unit.osc.v1 import fakes
from muranoclient.v1 import schemas as api_schemas

SAMPLE_CLASS_SCHEMA = {
    '': {
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {}
    },
    'modelBuilder': {
        "$schema": "http://json-schema.org/draft-04/schema#",
        "type": "object",
        "properties": {}
    }
}


class TestSchema(fakes.TestApplicationCatalog):
    def setUp(self):
        super(TestSchema, self).setUp()
        self.schemas_mock = \
            self.app.client_manager.application_catalog.schemas
        self.schemas_mock.get.return_value = api_schemas.Schema(
            None, SAMPLE_CLASS_SCHEMA)
        self.cmd = osc_schema.ShowSchema(self.app, None)

    def test_query_class_schema(self):
        arglist = ['class.name', 'methodName1',
                   '--package-name', 'package.name',
                   '--class-version', '>1']
        verifylist = [('class_name', 'class.name'),
                      ('method_names', ['methodName1']),
                      ('package_name', 'package.name'),
                      ('class_version', '>1')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        expected_columns = ['', 'modelBuilder']
        self.assertCountEqual(expected_columns, columns)
        self.assertCountEqual(tuple(SAMPLE_CLASS_SCHEMA.values()), data)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from muranoclient.osc import plugin
from muranoclient.tests.unit import base


class TestApplicationCatalogPlugin(base.TestCaseShell):

    @mock.patch("muranoclient.v1.client.Client")
    def test_make_client(self, p_client):

        instance = mock.Mock()
        instance._api_version = {"application_catalog": '1'}
        instance._region_name = 'murano_region'
        instance.session = 'murano_session'

        plugin.make_client(instance)
        p_client.assert_called_with(
            mock.ANY,
            region_name='murano_region',
            session='murano_session',
            service_type='application-catalog')



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import functools
import posixpath

from muranoclient.common import base


def normalize_path(f):
    @functools.wraps(f)
    def f_normalize_path(*args, **kwargs):
        path = args[2] if len(args) >= 3 else kwargs['path']

        # path formally is just absolute unix path
        if not posixpath.isabs(path):
            raise ValueError("Parameter 'path' should start with '/'")

        args = list(args)
        if len(args) >= 3:
            args[2] = args[2][1:]
        else:
            kwargs['path'] = kwargs['path'][1:]

        return f(*args, **kwargs)

    return f_normalize_path


class Service(base.Resource):
    def __repr__(self):
        return '<Service %s>' % self._info

    def data(self, **kwargs):
        return self.manager.data(self, **kwargs)

    def _add_details(self, info):
        if isinstance(info, dict):
            for k, v in info.items():
                setattr(self, k, v)


class ServiceManager(base.Manager):
    resource_class = Service

    def list(self, environment_id, session_id=None):
        if session_id:
            headers = {'X-Configuration-Session': session_id}
        else:
            headers = {}
        return self._list("/v1/environments/{0}/services".
                          format(environment_id), headers=headers)

    @normalize_path
    def get(self, environment_id, path, session_id=None):
        if session_id:
            headers = {'X-Configuration-Session': session_id}
        else:
            headers = {}

        return self._get('/v1/environments/{0}/services/{1}'.
                         format(environment_id, path), headers=headers)

    @normalize_path
    def post(self, environment_id, path, data, session_id):
        headers = {'X-Configuration-Session': session_id}

        result = self._create('/v1/environments/{0}/services/{1}'.
                              format(environment_id, path), data,
                              headers=headers, return_raw=True)

        if isinstance(result, list):
            return [self.resource_class(self, item) for item in result]
        else:
            return self.resource_class(self, result)

    @normalize_path
    def put(self, environment_id, path, data, session_id):
        headers = {'X-Configuration-Session': session_id}

        return self._update('/v1/environments/{0}/services/{1}'.
                            format(environment_id, path), data,
                            headers=headers)

    @normalize_path
    def delete(self, environment_id, path, session_id):
        headers = {'X-Configuration-Session': session_id}
        path = '/v1/environments/{0}/services/{1}'.format(environment_id, path)

        return self._delete(path, headers=headers)



#    Copyright (c) 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import shutil
import tempfile

import yaml

import muranoclient
from muranoclient.apiclient import exceptions
from muranoclient.common import utils


def prepare_package(args):
    """Prepare for application package

    Prepare all files and directories for that application package.
    Generates manifest file and all required parameters for that.

    :param args: list of command line arguments
    :returns: absolute path to directory with prepared files
    """
    if args.type and args.type not in ['Application', 'Library']:
        raise exceptions.CommandError(
            "--type should be set to 'Application' or 'Library'")

    manifest = generate_manifest(args)
    if args.type == 'Application':
        if not args.ui:
            raise exceptions.CommandError("'--ui' is required parameter")
        if not os.path.exists(args.ui) or not os.path.isfile(args.ui):
            raise exceptions.CommandError(
                "{0} is not a file or doesn`t exist".format(args.ui))

    temp_dir = tempfile.mkdtemp()
    manifest_file = os.path.join(temp_dir, 'manifest.yaml')
    classes_directory = os.path.join(temp_dir, 'Classes')
    resource_directory = os.path.join(temp_dir, 'Resources')

    with open(manifest_file, 'w') as f:
        f.write(yaml.dump(manifest, default_flow_style=False))

    logo_file = os.path.join(temp_dir, 'logo.png')
    if not args.logo or (args.logo and not os.path.isfile(args.logo)):
        shutil.copyfile(muranoclient.get_resource('mpl_logo.png'), logo_file)
    else:
        shutil.copyfile(args.logo, logo_file)

    shutil.copytree(args.classes_dir, classes_directory)
    if args.resources_dir:
        if not os.path.isdir(args.resources_dir):
            raise exceptions.CommandError(
                "'--resources-dir' parameter should be a directory")
        shutil.copytree(args.resources_dir, resource_directory)
    if args.ui:
        ui_directory = os.path.join(temp_dir, 'UI')
        os.mkdir(ui_directory)
        shutil.copyfile(args.ui, os.path.join(ui_directory, 'ui.yaml'))
    return temp_dir


def generate_manifest(args):
    """Generates application manifest file.

    If some parameters are missed - they we be generated automatically.
    :param args:
    :returns: dictionary, contains manifest file data
    """
    if not os.path.isdir(args.classes_dir):
        raise exceptions.CommandError(
            "'--classes-dir' parameter should be a directory")
    args = update_args(args)
    if not args.type:
        raise exceptions.CommandError(
            "Too few arguments: --type and --full-name is required")

    if not args.author:
        args.author = args.os_username
    if not args.description:
        args.description = "Description for the application is not provided"

    if not args.full_name:
        raise exceptions.CommandError(
            "Please, provide --full-name parameter")

    manifest = {
        'Format': 'MuranoPL/1.0',
        'Type': args.type,
        'FullName': args.full_name,
        'Name': args.name,
        'Description': args.description,
        'Author': args.author,
        'Classes': args.classes
    }

    if args.tags:
        manifest['Tags'] = args.tags
    return manifest


def update_args(args):
    """Add and update arguments if possible.

    Some parameters are not required and would be guessed
    from muranoPL classes: thus, if class extends system application class
    fully qualified and require names could be calculated.
    Also, in that case type of a package could be set to 'Application'.
    """

    classes = {}
    extends_from_application = False
    for root, dirs, files in os.walk(args.classes_dir):
        for class_file in files:
            class_file_path = os.path.join(root, class_file)
            try:
                with open(class_file_path) as f:
                    content = yaml.load(f, utils.YaqlYamlLoader)

                if not content.get('Name'):
                    raise exceptions.CommandError(
                        "Error in class definition: 'Name' "
                        "section is required")
                class_name = get_fqn_for_name(content.get('Namespaces'),
                                              content['Name'])
                if root == args.classes_dir:
                    relative_path = class_file
                else:
                    relative_path = os.path.join(
                        root.replace(args.classes_dir, "")[1:],
                        class_file)
                classes[class_name] = relative_path

                extends_from_application = check_derived_from_application(
                    content, extends_from_application)
                if extends_from_application:
                    if not args.type:
                        args.type = 'Application'
                    if not args.name:
                        args.name = class_name.split('.')[-1]
                    if not args.full_name:
                        args.full_name = class_name

            except yaml.YAMLError:
                raise exceptions.CommandError(
                    "MuranoPL class {0} should be"
                    " a valid yaml file".format(class_file_path))
            except IOError:
                raise exceptions.CommandError(
                    "Could not open file {0}".format(class_file_path))
    if not classes:
        raise exceptions.CommandError("Application should have "
                                      "at least one class")
    args.classes = classes
    return args


def get_fqn_for_name(namespaces, name):
    """Analyze name for namespace reference.

    If namespaces are used - return a full name
    :param namespaces: content of 'Namespaces' section of muranoPL class
    :param name: name that should be checked
    :returns: generated name according to namespaces
    """
    values = name.split(':')
    if len(values) == 1:
        if '=' in namespaces:
            return namespaces['='] + '.' + values[0]
        return values[0]
    if len(values) > 2:
        raise exceptions.CommandError(
            "Error in class definition: Wrong usage of ':' is "
            "reserved for namespace referencing and could "
            "be used only once "
            "for each name")
    if not namespaces:
        raise exceptions.CommandError(
            "Error in {0} class definition: "
            "'Namespaces' section is missed")

    result = namespaces.get(values[0])
    if not result:
        raise exceptions.CommandError(
            "Error in class definition: namespaces "
            "reference is not correct at the 'Extends'"
            " section")
    return result + '.' + values[1]


def check_derived_from_application(content, extends_from_application):
    """Look up for system 'io.murano.Application' class in extends section"""
    if content.get('Extends'):
        extends = content['Extends']
        if not isinstance(extends, list):
            extends = [extends]

        for name in extends:
            parent_class_name = get_fqn_for_name(
                content.get('Namespaces'),
                name)
            if parent_class_name == 'io.murano.Application':
                if not extends_from_application:
                    return True
                else:
                    raise exceptions.CommandError(
                        "Murano package should have only one class"
                        " extends 'io.murano.Application' class")
    return False



#    Copyright (c) 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils
import urllib
import yaml

from muranoclient.common import base
from muranoclient.common import exceptions
from muranoclient.common import utils

DEFAULT_PAGE_SIZE = 20


class Package(base.Resource):
    def __repr__(self):
        return "<Package %s>" % self._info

    def data(self, **kwargs):
        return self.manager.data(self, **kwargs)


class PackageManager(base.Manager):
    resource_class = Package
    _tracked_packages = set()

    def create(self, data, files):
        for pkg_file in files.values():
            utils.Package.from_file(pkg_file)
            pkg_file.seek(0)

        response = self.api.request(
            '/v1/catalog/packages',
            'POST',
            data={'__metadata__': jsonutils.dumps(data)},
            files=files
        )
        if not response.ok:
            setattr(response, 'status', response.status_code)
            raise exceptions.from_response(response)
        body = jsonutils.loads(response.text)
        return self.resource_class(self, body)

    def get(self, app_id):
        return self._get('/v1/catalog/packages/{0}'.format(app_id))

    def filter(self, **kwargs):

        def construct_url(params):
            for k, v in params.items():
                if isinstance(v, str):
                    v = v.encode('utf-8')
                    params[k] = v
            return '?'.join(
                ['/v1/catalog/packages',
                 urllib.parse.urlencode(params, doseq=True)]
            )

        def paginate(_url):
            # code from Glance
            resp, body = self.api.json_request(_url, 'GET')
            for package in body['packages']:
                yield package
            try:
                m_kwargs = kwargs.copy()
                m_kwargs['marker'] = body['next_marker']
                next_url = construct_url(m_kwargs)
            except KeyError:
                return
            else:
                for package in paginate(next_url):
                    yield package

        if 'page_size' not in kwargs:
            kwargs['limit'] = kwargs.get('limit', DEFAULT_PAGE_SIZE)
        else:
            kwargs['limit'] = kwargs['page_size']

        url = construct_url(kwargs)

        for package in paginate(url):
            yield self.resource_class(self, package, loaded=True)

    def list(self, include_disabled=False, limit=20):
        return self.filter(include_disabled=include_disabled, limit=limit)

    def delete(self, app_id):
        return self._delete('/v1/catalog/packages/{0}'.format(app_id))

    def update(self, app_id, body, operation='replace'):
        """Translates dictionary to jsonpatch request

        :param app_id: string, id of updating application
        :param body: dictionary, mapping between keys and values for update
        :param operation: string, way of updating: replace, remove, add
        :returns: HTTP response
        """
        url = '/v1/catalog/packages/{0}'.format(app_id)
        data = []
        for key, value in body.items():
            data.append({'op': operation, 'path': '/' + key, 'value': value})
        return self.api.json_patch_request(url, data=data)

    def download(self, app_id):
        url = '/v1/catalog/packages/{0}/download'.format(app_id)
        response = self.api.request(url, 'GET', log=False)
        if response.status_code == 200:
            return response.content
        else:
            raise exceptions.from_response(response)

    def toggle_active(self, app_id):
        url = '/v1/catalog/packages/{0}'.format(app_id)
        enabled = self.get(app_id).enabled
        data = [{'op': 'replace', 'path': '/enabled', 'value': not enabled}]
        return self.api.json_patch_request(url, data=data)

    def toggle_public(self, app_id):
        url = '/v1/catalog/packages/{0}'.format(app_id)
        is_public = self.get(app_id).is_public
        data = [{'op': 'replace', 'path': '/is_public',
                'value': not is_public}]
        return self.api.json_patch_request(url, body=data)

    def get_ui(self, app_id, loader_cls=None):
        if loader_cls is None:
            loader_cls = yaml.SafeLoader

        url = '/v1/catalog/packages/{0}/ui'.format(app_id)
        response = self.api.request(url, 'GET')
        if response.status_code == 200:
            return yaml.load(response.content, loader_cls)
        else:
            raise exceptions.from_response(response)

    def get_logo(self, app_id):
        url = '/v1/catalog/packages/{0}/logo'.format(app_id)
        response = self.api.request(url, 'GET')
        if response.status_code == 200:
            return response.content
        else:
            raise exceptions.from_response(response)

    def get_supplier_logo(self, app_id):
        url = '/v1/catalog/packages/{0}/supplier_logo'.format(app_id)
        response = self.api.request(url, 'GET')
        if response.status_code == 200:
            return response.content
        else:
            raise exceptions.from_response(response)



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import urllib

from muranoclient.common import base


class Schema(base.Resource):
    def __repr__(self):
        return "<Schema %s>" % self._info

    @property
    def data(self):
        return self._info


class SchemaManager(base.Manager):
    resource_class = Schema

    def get(self, class_name, method_names=None,
            class_version=None, package_name=None):
        """Get JSON-schema for class or method"""

        if isinstance(method_names, (list, tuple)):
            method_names = ','.join(method_names)

        base_url = '/v1/schemas/' + '/'.join(
            t for t in (class_name, method_names) if t)

        params = {
            key: value for key, value in (
                ('classVersion', class_version),
                ('packageName', package_name)) if value
        }

        if len(params):
            base_url += '?' + urllib.parse.urlencode(params, True)

        return self._get(base_url)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Application-catalog v1 class-schema action implementation"""

from osc_lib.command import command
from oslo_log import log as logging

LOG = logging.getLogger(__name__)


class ShowSchema(command.ShowOne):
    """Show class schema."""

    def get_parser(self, prog_name):
        parser = super(ShowSchema, self).get_parser(prog_name)
        parser.add_argument(
            "class_name", metavar="<CLASS>", help="Class FQN")
        parser.add_argument(
            "method_names", metavar="<METHOD>",
            help="Method name", nargs='*')
        parser.add_argument(
            "--package-name", default=None,
            help="FQN of the package where the class is located")
        parser.add_argument(
            "--class-version", default='=0',
            help="Class version or version range (version spec)")

        return parser

    def take_action(self, parsed_args):
        LOG.debug("take_action({0})".format(parsed_args))
        client = self.app.client_manager.application_catalog
        schema = client.schemas.get(
            parsed_args.class_name, parsed_args.method_names,
            class_version=parsed_args.class_version,
            package_name=parsed_args.package_name)

        return self.dict2columns(schema.data)

    @property
    def formatter_default(self):
        return 'json'



#    Copyright (c) 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import shutil
import tempfile

import yaml

import muranoclient
from muranoclient.apiclient import exceptions


def generate_manifest(args):
    """Generates application manifest file.

    If some parameters are missed - they we be generated automatically.
    :param args:
    :returns: dictionary, contains manifest file data
    """
    if not os.path.isfile(args.template):
        raise exceptions.CommandError(
            "Template '{0}' doesn`t exist".format(args.template))
    filename = os.path.basename(args.template)
    if not args.name:
        args.name = os.path.splitext(filename)[0]
    if not args.full_name:
        prefix = 'io.murano.apps.generated'
        normalized_name = args.name.replace('_', ' ').replace('-', ' ')
        normalized_name = normalized_name.title().replace(' ', '')
        args.full_name = '{0}.{1}'.format(prefix, normalized_name)
    try:
        with open(args.template, 'rb') as heat_file:
            yaml_content = yaml.safe_load(heat_file)
            if not args.description:
                args.description = yaml_content.get(
                    'description',
                    'Heat-defined application for a template "{0}"'.format(
                        filename))
    except yaml.YAMLError:
        raise exceptions.CommandError(
            "Heat template, represented by --'template' parameter"
            " should be a valid yaml file")
    if not args.author:
        args.author = args.os_username
    if not args.tags:
        args.tags = ['Heat-generated']

    manifest = {
        'Format': 'Heat.HOT/1.0',
        'Type': 'Application',
        'FullName': args.full_name,
        'Name': args.name,
        'Description': args.description,
        'Author': args.author,
        'Tags': args.tags
    }
    return manifest


def prepare_package(args):
    """Compose required files for murano application package.

    :param args: list of command line arguments
    :returns: absolute path to directory with prepared files
    """
    manifest = generate_manifest(args)

    temp_dir = tempfile.mkdtemp()
    manifest_file = os.path.join(temp_dir, 'manifest.yaml')
    template_file = os.path.join(temp_dir, 'template.yaml')

    if args.resources_dir:
        if not os.path.isdir(args.resources_dir):
            raise exceptions.CommandError(
                "'--resources-dir' parameter should be a directory")
        resource_directory = os.path.join(temp_dir, 'Resources')
        shutil.copytree(args.resources_dir, resource_directory)

    logo_file = os.path.join(temp_dir, 'logo.png')
    if not args.logo:
        shutil.copyfile(muranoclient.get_resource('heat_logo.png'), logo_file)
    else:
        if os.path.isfile(args.logo):
            shutil.copyfile(args.logo, logo_file)

    with open(manifest_file, 'w') as f:
        f.write(yaml.dump(manifest, default_flow_style=False))
    shutil.copyfile(args.template, template_file)

    return temp_dir



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from muranoclient.osc.v1 import deployment as osc_deployment
from muranoclient.tests.unit.osc.v1 import fakes
from muranoclient.v1 import deployments as api_deployment

DEPLOYMENT_COLUMNS = ('id', 'state', 'created', 'updated', 'finished')
DEPLOYMENT_DATA = ('xyz123', 'success', '2016-06-25T12:21:37',
                   '2016-06-25T12:21:47', '2016-06-25T12:21:47')
ALL_DEPLOYMENT_DATA = (('abc123', 'success', '2016-06-25T12:21:37',
                        '2016-06-25T12:21:47', '2016-06-25T12:21:47'),
                       ('xyz456', 'success', '2017-01-31T11:22:35',
                        '2017-01-31T11:22:47', '2017-01-31T11:22:47'))


class TestDeployment(fakes.TestApplicationCatalog):
    def setUp(self):
        super(TestDeployment, self).setUp()
        self.deployment_mock = self.app.client_manager.application_catalog.\
            deployments
        self.deployment_mock.reset_mock()
        self.environment_mock = self.app.client_manager.application_catalog.\
            environments


class TestListDeployment(TestDeployment):
    def setUp(self):
        super(TestListDeployment, self).setUp()
        deployment_info = dict(zip(DEPLOYMENT_COLUMNS, DEPLOYMENT_DATA))
        self.deployment_mock.list.return_value = \
            [api_deployment.Deployment(None, deployment_info)]

        # Command to test
        self.cmd = osc_deployment.ListDeployment(self.app, None)

    @mock.patch('osc_lib.utils.get_item_properties')
    def test_deployment_list(self, mock_util):
        arglist = ['xyz123']
        verifylist = [('id', 'xyz123')]
        mock_util.return_value = DEPLOYMENT_DATA

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = [c.title() for c in DEPLOYMENT_COLUMNS]
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [DEPLOYMENT_DATA]
        self.assertEqual(expected_data, data)

    @mock.patch('osc_lib.utils.get_item_properties', autospec=True)
    def test_deployment_list_all_environments(self, mock_util):
        arglist = ['--all-environments']
        verifylist = [('id', None), ('all_environments', True)]
        mock_util.return_value = ALL_DEPLOYMENT_DATA

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = [c.title() for c in DEPLOYMENT_COLUMNS]
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [ALL_DEPLOYMENT_DATA]
        self.assertEqual(expected_data, data)



#    Copyright (c) 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import shutil

from muranoclient.apiclient import exceptions
from muranoclient.tests.unit import base
from muranoclient.v1.package_creator import hot_package
from muranoclient.v1.package_creator import mpl_package


FIXTURE_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                           'fixture_data'))
TEMPLATE = os.path.join(FIXTURE_DIR, 'heat-template.yaml')
CLASSES_DIR = os.path.join(FIXTURE_DIR, 'test-app', 'Classes')
RESOURCES_DIR = os.path.join(FIXTURE_DIR, 'test-app', 'Resources')
UI = os.path.join(FIXTURE_DIR, 'test-app', 'ui.yaml')
LOGO = os.path.join(FIXTURE_DIR, 'logo.png')


class TestArgs(object):
    pass


class PackageCreatorTest(base.TestAdditionalAsserts):

    def test_generate_hot_manifest(self):
        args = TestArgs()
        args.template = TEMPLATE
        args.name = 'test_name'
        args.author = 'TestAuthor'
        args.full_name = None
        args.tags = None
        args.description = None
        expected_manifest = {
            'Format': 'Heat.HOT/1.0',
            'Type': 'Application',
            'FullName': 'io.murano.apps.generated.TestName',
            'Name': 'test_name',
            'Description': 'Heat-defined application '
                           'for a template "heat-template.yaml"',
            'Author': 'TestAuthor',
            'Tags': ['Heat-generated']
        }
        result_manifest = hot_package.generate_manifest(args)
        self.check_dict_is_subset(expected_manifest, result_manifest)

    def test_generate_hot_manifest_nonexistent_template(self):
        args = TestArgs()
        args.template = '/home/this/path/does/not/exist'
        self.assertRaises(exceptions.CommandError,
                          hot_package.generate_manifest,
                          args)

    def test_generate_hot_manifest_with_all_parameters(self):
        args = TestArgs()
        args.template = TEMPLATE
        args.name = 'test_name'
        args.author = 'TestAuthor'
        args.full_name = 'test.full.name.TestName'
        args.tags = ['test', 'tag', 'Heat']
        args.description = 'Test description'

        expected_manifest = {
            'Format': 'Heat.HOT/1.0',
            'Type': 'Application',
            'FullName': 'test.full.name.TestName',
            'Name': 'test_name',
            'Description': 'Test description',
            'Author': 'TestAuthor',
            'Tags': ['test', 'tag', 'Heat']
        }
        result_manifest = hot_package.generate_manifest(args)
        self.check_dict_is_subset(expected_manifest, result_manifest)

    def test_generate_hot_manifest_template_not_yaml(self):
        args = TestArgs()
        args.template = LOGO
        args.name = None
        args.full_name = None
        self.assertRaises(exceptions.CommandError,
                          hot_package.generate_manifest, args)

    def test_prepare_hot_package(self):
        args = TestArgs()
        args.template = TEMPLATE
        args.name = 'test_name'
        args.author = 'TestAuthor'
        args.full_name = 'test.full.name.TestName'
        args.tags = 'test, tag, Heat'
        args.description = 'Test description'
        args.resources_dir = RESOURCES_DIR
        args.logo = LOGO
        package_dir = hot_package.prepare_package(args)

        prepared_files = ['manifest.yaml', 'logo.png',
                          'template.yaml', 'Resources']
        self.assertEqual(sorted(prepared_files),
                         sorted(os.listdir(package_dir)))
        shutil.rmtree(package_dir)

    def test_generate_mpl_manifest(self):
        args = TestArgs()
        args.template = TEMPLATE
        args.classes_dir = CLASSES_DIR
        args.resources_dir = RESOURCES_DIR
        args.type = 'Application'
        args.author = 'TestAuthor'
        args.name = None
        args.full_name = None
        args.tags = None
        args.description = None

        expected_manifest = {
            'Format': 'MuranoPL/1.0',
            'Type': 'Application',
            'Classes': {'io.murano.apps.test.APP': 'testapp.yaml'},
            'FullName': 'io.murano.apps.test.APP',
            'Name': 'APP',
            'Description': 'Description for the application is not provided',
            'Author': 'TestAuthor',
        }
        result_manifest = mpl_package.generate_manifest(args)
        self.check_dict_is_subset(expected_manifest, result_manifest)

    def test_generate_mpl_manifest_with_all_parameters(self):
        args = TestArgs()
        args.template = TEMPLATE
        args.classes_dir = CLASSES_DIR
        args.resources_dir = RESOURCES_DIR
        args.type = 'Application'
        args.name = 'test_name'
        args.author = 'TestAuthor'
        args.full_name = 'test.full.name.TestName'
        args.tags = ['test', 'tag', 'Heat']
        args.description = 'Test description'

        expected_manifest = {
            'Format': 'MuranoPL/1.0',
            'Type': 'Application',
            'Classes': {'io.murano.apps.test.APP': 'testapp.yaml'},
            'FullName': 'test.full.name.TestName',
            'Name': 'test_name',
            'Description': 'Test description',
            'Author': 'TestAuthor',
            'Tags': ['test', 'tag', 'Heat']
        }
        result_manifest = mpl_package.generate_manifest(args)
        self.check_dict_is_subset(expected_manifest, result_manifest)

    def test_generate_mpl_wrong_classes_dir(self):
        args = TestArgs()
        args.classes_dir = '/home/this/path/does/not/exist'
        expected = ("'--classes-dir' parameter should be a directory", )
        try:
            mpl_package.generate_manifest(args)
        except exceptions.CommandError as message:
            self.assertEqual(expected, message.args)

    def test_prepare_mpl_wrong_resources_dir(self):
        args = TestArgs()
        args.template = TEMPLATE
        args.classes_dir = CLASSES_DIR
        args.resources_dir = '/home/this/path/does/not/exist'
        args.type = 'Application'
        args.name = 'Test'
        args.tags = ''
        args.ui = UI
        args.logo = LOGO
        args.full_name = 'test.full.name.TestName'
        args.author = 'TestAuthor'
        args.description = 'Test description'

        expected = ("'--resources-dir' parameter should be a directory", )
        try:
            mpl_package.prepare_package(args)
        except exceptions.CommandError as message:
            self.assertEqual(expected, message.args)

    def test_prepare_mpl_package(self):
        args = TestArgs()
        args.template = TEMPLATE
        args.classes_dir = CLASSES_DIR
        args.resources_dir = RESOURCES_DIR
        args.type = 'Application'
        args.name = 'test_name'
        args.author = 'TestAuthor'
        args.full_name = 'test.full.name.TestName'
        args.tags = 'test, tag, Heat'
        args.description = 'Test description'
        args.ui = UI
        args.logo = LOGO
        prepared_files = ['UI', 'Classes', 'manifest.yaml',
                          'Resources', 'logo.png']
        package_dir = mpl_package.prepare_package(args)
        self.assertEqual(sorted(prepared_files),
                         sorted(os.listdir(package_dir)))
        shutil.rmtree(package_dir)



# Copyright (c) 2014 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import configparser
import os

from tempest.lib.cli import base


class ClientTestBase(base.ClientTestBase):

    def murano(self, action, flags='', params='',
               fail_ok=False, endpoint_type='publicURL', merge_stderr=True):
        flags += self.get_backend_flag()
        return self.clients.cmd_with_auth(
            'murano', action, flags, params, fail_ok, merge_stderr)

    def _get_clients(self):
        cli_dir = os.environ.get(
            'OS_MURANOCLIENT_EXEC_DIR',
            os.path.join(os.path.abspath('.'), '.tox/functional/bin'))

        self.username = os.environ.get('OS_USERNAME')
        self.password = os.environ.get('OS_PASSWORD')
        self.tenant_name = os.environ.get('OS_PROJECT_NAME',
                                          os.environ.get('OS_TENANT_NAME'))
        self.uri = os.environ.get('OS_AUTH_URL')
        config = configparser.RawConfigParser()
        if config.read('functional_creds.conf'):
            # the OR pattern means the environment is preferred for
            # override
            self.username = self.username or config.get('admin', 'user')
            self.password = self.password or config.get('admin', 'pass')
            self.tenant_name = self.tenant_name or config.get('admin',
                                                              'tenant')
            self.uri = self.uri or config.get('auth', 'uri')

        clients = base.CLIClient(
            username=self.username,
            password=self.password,
            tenant_name=self.tenant_name,
            uri=self.uri,
            cli_dir=cli_dir
        )
        return clients

    def listing(self, command, params=""):
        return self.parser.listing(self.murano(command, params=params))

    def get_value(self, need_field, known_field, known_value, somelist):
        for element in somelist:
            if element[known_field] == known_value:
                return element[need_field]

    @staticmethod
    def get_backend_flag():
        backend = os.environ.get('MURANO_PACKAGES_SERVICE', 'murano')
        backend_flag = " --murano-packages-service {0} ".format(backend)
        return backend_flag



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from muranoclient.osc.v1 import action as osc_action
from muranoclient.tests.unit.osc.v1 import fakes
from muranoclient.v1 import static_actions as api_static_actions


class TestAction(fakes.TestApplicationCatalog):
    def setUp(self):
        super(TestAction, self).setUp()
        self.static_actions_mock = \
            self.app.client_manager.application_catalog.static_actions


class TestStaticActionCall(TestAction):
    def setUp(self):
        super(TestStaticActionCall, self).setUp()
        self.static_actions_mock.call.return_value = \
            api_static_actions.StaticActionResult('result')

        # Command to test
        self.cmd = osc_action.StaticActionCall(self.app, None)

    @mock.patch('osc_lib.utils.get_item_properties')
    def test_static_action_call_basic(self, mock_util):
        mock_util.return_value = 'result'

        arglist = ['class.name', 'method.name']
        verifylist = [('class_name', 'class.name'),
                      ('method_name', 'method.name')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Static action result']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ['result']
        self.assertEqual(expected_data, data)

    @mock.patch('osc_lib.utils.get_item_properties')
    def test_static_action_call_full(self, mock_util):
        mock_util.return_value = 'result'

        arglist = ['class.name', 'method.name',
                   '--arguments', 'food=spam', 'parrot=dead',
                   '--package-name', 'package.name',
                   '--class-version', '>1']
        verifylist = [('class_name', 'class.name'),
                      ('method_name', 'method.name'),
                      ('arguments', ['food=spam', 'parrot=dead']),
                      ('package_name', 'package.name'),
                      ('class_version', '>1')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = ['Static action result']
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = ['result']
        self.assertEqual(expected_data, data)
